/*
 * TestJointControlCTM.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package example;

import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.controllers.ComputedTorque;
import javax.robotics.engine.controllers.JointController;
import javax.robotics.engine.robots.DHRobot;
import javax.robotics.engine.robots.Robot;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.engine.trajectory.Trapezoidal;
import javax.robotics.ode.DerivableFunction2;
import javax.robotics.ode.RungeKutta4th2;
import javax.robotics.ode.Solver2;
import javax.robotics.plot.MultiPlot;
import javax.robotics.plot.MultiPlotFrame;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;

import org.apache.xmlbeans.XmlException;
/**
 * Solution of problem 6.12 of Modelling and Control of Robot Manipulators by
 * Sciavicco, Siciliano(2nd Edition) (compensated load mass)
 * 
 * @author Carmine Lia
 * 
 */
public class TestJointControlCTM implements DerivableFunction2
{
    private final ClassLoader cl = this.getClass().getClassLoader();
    MultiPlotFrame frame;
    MultiPlot plotErrPos, plotPos, plotVel, plotTorque;
    // numero dei giunti del robot
    int dof, npoints;
    double dur;
    RVector q, qp, qd, qpd, qppd, tau, qs, qf, qpc, tf, err;
    Robot robot;
    Solver2 solver;
    Trajectory trj;
    JointController ctm;

    public TestJointControlCTM() throws XmlException, IOException, URISyntaxException
    {

        dur = 2.50;
        double dt = 0.001;
        npoints = 2500;

        err = new RVector(2);
        q = new RVector(2);
        qp = new RVector(2);
        qd = new RVector(2);
        qpd = new RVector(2);
        qppd = new RVector(2);
        tau = new RVector(2);

        // initial joints position
        qs = new RVector(0, Math.PI / 4);
        // final joints position
        qf = new RVector(Math.PI / 2, Math.PI / 2);
        // max joints velocity
        qpc = new RVector(3 * Math.PI / 4, Math.PI / 3);

        // init trajectory
        trj = new Trapezoidal(qs, qf, qpc, 1);

        // init solver with initial state
        solver = new RungeKutta4th2(dt, new RVector(qs), new RVector(2), this);

        // init robot setting the joint position at time 0
        robot = new DHRobot(cl.getResourceAsStream("example/conf/rr_book_chap6_load.xml"));
        dof = robot.getDof();
        robot.setQ(qs);

        // set the gravity vector
        // System.out.println("Gravity vector:\n"+robot.getGravity());
        robot.setGravity(new RVector3d(0.0, 9.81, 0.0));
        System.out.println("Gravity vector:\n" + robot.getGravity());

        // init controller
        ctm = new ComputedTorque(robot, new DMatrix(100, 100), new DMatrix(16,
                16));

        createPlotPanel();

        runControl();

    }
    private void createPlotPanel()
    {
        frame = new  MultiPlotFrame("CTM test");
    
        plotErrPos = new MultiPlot("Position error", "[s]", "[rad]", "joint 1",
                "joint 2");// ,"desired 1", "desired 2");
        plotErrPos.setXRange(0, dur);

        plotPos = new MultiPlot("Position", "[s]", "[rad]", "joint 1", "joint 2");
        plotPos.setXRange(0, dur);
        plotPos.setYRange(0, 1.6);

        plotVel = new MultiPlot("Velocity", "[s]", "[rad/s]", "joint 1", "joint 2");
        plotVel.setXRange(0, dur);

        plotTorque = new MultiPlot("Torque", "[s]", "[Nm]", "joint 1", "joint 2");
        plotTorque.setXRange(0, dur);

        frame.add(plotErrPos);
        frame.add(plotPos);
        frame.add(plotVel);
        frame.add(plotTorque);

        frame.setDefaultCloseOperation(MultiPlotFrame.EXIT_ON_CLOSE);
        frame.create();
        return;
    }

    private void runControl()
    {
        double time;

        for (int i = 0; i < npoints; i++)
        {

            time = solver.getTime();
            q = solver.getLastValuePosition();
            qp = solver.getLastValueVelocity();

            // joints path interpolation
            trj.interpolate(time, qd, qpd, qppd);

            tau = ctm.torqueCmd(qd, qpd, qppd);

            err.sub(qd, q);
            plotErrPos.addVector(time, err, true);
            plotPos.addVector(time, q, true);
            plotVel.addVector(time, qp, true);
            plotTorque.addVector(time, tau, true);

            solver.step(q, qp);

        }
        plotErrPos.fillPlot();
        plotPos.fillPlot();
        plotVel.fillPlot();
        plotTorque.fillPlot();
    }
    // implementation of derivable function of Solver
    public RVector xDot2(RVector q, RVector qp)
    {
        return robot.acceleration(q, qp, tau, new RVector3d(0, 0, 0),
                new RVector3d());
    }
    /**
     * @param args
     * @throws IOException 
     * @throws XmlException 
     * @throws URISyntaxException 
     */
    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new TestJointControlCTM();
    }

}
