/*
 * TestJointControlPDGravity.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package example;

import java.io.IOException;
import java.net.URISyntaxException;

import javax.robotics.engine.controllers.JointController;
import javax.robotics.engine.controllers.PDGravityCompensation;
import javax.robotics.engine.robots.DHRobot;
import javax.robotics.engine.robots.Robot;
import javax.robotics.ode.DerivableFunction2;
import javax.robotics.ode.RungeKutta4th2;
import javax.robotics.ode.Solver2;
import javax.robotics.plot.MultiPlot;
import javax.robotics.plot.MultiPlotFrame;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;

import org.apache.xmlbeans.XmlException;
/**
 * Solution of problem 6.10 of Modelling and Control of Robot Manipulators by
 * Sciavicco, Siciliano(2nd Edition)
 * 
 * @author Carmine Lia
 * 
 */
public class TestJointControlPDGravity implements DerivableFunction2
{
    private final ClassLoader cl = this.getClass().getClassLoader();
    MultiPlotFrame frame;
    MultiPlot plotPos, plotVel, plotAcc, plotTorque;
    // numero dei giunti del robot
    int dof, npoints;
    double dur;
    RVector q, qp, qpp, qd, tau, q0;
    Robot robot;
    Solver2 solver;
    JointController pdg;

    public TestJointControlPDGravity() throws XmlException, IOException, URISyntaxException
    {
        dur = 2.5;
        double dt = 0.001;
        npoints = 2500;

        q = new RVector(2);
        qp = new RVector(2);
        qpp = new RVector(2);
        tau = new RVector(2);

        // desired joints position
        qd = new RVector(-Math.PI, -3 * Math.PI / 4);

        // intial joints position
        q0 = new RVector(-Math.PI - 0.1, -3 * Math.PI / 4 - 0.1);

        // init solver solver with initial state
        // solver = new Euler2(dt, new RVector(-Math.PI-0.1, -3*Math.PI/4-0.1),
        // new RVector(2), this);
        // init solver runge-kutta4 with initial state
        solver = new RungeKutta4th2(dt, q0, new RVector(2), this);

        // init robot setting the joint position at time 0
        robot = new DHRobot(cl.getResourceAsStream("example/conf/rr_book_chap6.xml"));
        dof = robot.getDof();
        robot.setQ(q0);
        robot.setGravity(new RVector3d(0.0, 9.81, 0.0));
        System.out.println("Gravity vector:\n" + robot.getGravity());

        // init controller
        pdg = new PDGravityCompensation(robot, new DMatrix(3750, 3750),
                new DMatrix(750, 750));

        createPlotPanel();

        runControl();

    }
    private void createPlotPanel()
    {
        frame = new MultiPlotFrame("Proportional Derivative with Gravity compensation", 2, 2);
       
        plotPos = new MultiPlot("Position", "[s]", "[rad]", "joint 1", "joint 2",
                "desired 1", "desired 2");
        plotPos.setXRange(0, 2.5);
        plotPos.setYRange(-3, 1);
        plotVel = new MultiPlot("Velocity", "[s]", "[rad/s]", "joint 1", "joint 2");
        plotVel.setXRange(0, 2.5);
        plotAcc = new MultiPlot("Acceleration", "[s]", "[rad/s^2]", "joint 1",
                "joint 2");
        plotAcc.setXRange(0, 2.5);
        plotTorque = new MultiPlot("Torque", "[s]", "[Nm]", "joint 1", "joint 2");
        plotTorque.setXRange(0, 2.5);

        frame.add(plotPos);
        frame.add(plotVel);
        frame.add(plotAcc);
        frame.add(plotTorque);

        frame.setDefaultCloseOperation(MultiPlotFrame.EXIT_ON_CLOSE);
        frame.create();
        return;
    }

    private void runControl()
    {
        double time;

        for (int i = 0; i < npoints; i++)
        {

            time = solver.getTime();
            q = solver.getLastValuePosition();
            qp = solver.getLastValueVelocity();

            tau = pdg.torqueCmd(qd);

            plotPos.addVector(0, time, q, true);
            plotPos.addVector(2, time, qd, true);

            plotVel.addVector(time, qp, true);
            plotTorque.addVector(time, tau, true);

            solver.step(q, qp);
            plotAcc.addVector(time, qpp, true);

        }
        plotPos.fillPlot();
        plotVel.fillPlot();
        plotAcc.fillPlot();
        plotTorque.fillPlot();

    }
    // implementation of derivable function of Solver
    public RVector xDot2(RVector q, RVector qp)
    {

        qpp = robot.acceleration(q, qp, tau);

        return qpp;
    }
    /**
     * @param args
     * @throws IOException 
     * @throws XmlException 
     * @throws URISyntaxException 
     */
    public static void main(String[] args) throws XmlException, IOException, URISyntaxException
    {
        new TestJointControlPDGravity();
    }

}
