/*
 * CLIK.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;

/**
 * Closed Loop Inverse Kinematic algorithms interface for robots manipulator.
 * 
 * @author Carmine Lia
 * @since 1.0.2
 * @version 11/11/2005
 * 
 */
public interface CLIK
{
    /**
     * Gets the singular region area.
     * 
     * @return the eps.
     */
    public double getEps();
    /**
     * Sets the singular region area.
     * 
     * @param eps
     *            The eps to set.
     */
    public void setEps(double eps);

    /**
     * Gets the sample time.
     * 
     * @return the sample time.
     */
    public double getSampleTime();

    /**
     * Sets the sample time.
     * 
     * @param h
     *            The sample time to set.
     */
    public void setSampleTime(double h);

    /**
     * Gets the matrix of error position gain.
     * 
     * @return the diagonal matrix.
     */
    public DMatrix getKp();

    /**
     * Sets the matrix of error position gain.
     * 
     * @param kp
     *            The diagonal matrix to set.
     */
    public void setKp(DMatrix kp);

    /**
     * Gets the matrix of error orientation gain.
     * 
     * @return the diagonal matrix.
     */
    public DMatrix getKo();

    /**
     * Sets the matrix of error orientation gain.
     * 
     * @param kd
     *            The diagonal matrix to set.
     */
    public void setKo(DMatrix kd);

    /**
     * Gets the maximum damped factor.
     * 
     * @return the damped factor.
     */
    public double getLambdaMax();

    /**
     * Sets the maximum damped factor.
     * 
     * @param lambdaMax
     *            The damped factor to set.
     */
    public void setLambdaMax(double lambdaMax);

    /**
     * Gets the current position error.
     * 
     * @return the position error vector.
     */
    public RVector3d getPositionError();
    /**
     * Gets the current orientation error.
     * The error is computed form quaternion as:
     * <p>
     * &eta;(q)&epsilon<sub>d</sub> - &eta<sub>d</sub>&epsilon(q) -
     * S(&epsilon<sub>d</sub>)&epsilon(q)
     * </p> where &eta; and &epsilon; are the vector and the scalar components of quaternion respectively.
     * @return the orientation error vector.
     */
    public RVector3d getOrientationError();
   
    /**
     * Gets the current joints position.
     * @return the joints position vector.
     */
    public RVector getJointsPosition();
    
    /**
     * Gets the current joints velocity.
     * @return the joints velocity vector.
     */
    public RVector getJointsVelocity();  
    
    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and orientation.
     * 
     * @param pd
     *            the vector of desired end effector position.
     * @param ppd
     *            the vector of desired end effector velocity.
     * @param quatd
     *            the desired end effector quaternion.
     * @param wd
     *            the desired end effector angular velocity.
     */
    public void step(RVector3d pd, RVector3d ppd, Quaternion quatd,
            RVector3d wd);

}
