/*
 * CLIKR.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;

/**
 * Closed Loop Inverse Kinematic algorithms interface for redundant robots
 * manipulator.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.2
 * @version 11/11/2005
 */
public interface CLIKR
{
    /**
     * Gets the dimension of cartesian space.
     * For planar robots the value is 2, the value is set from dimension of diagonal matrix gain Kp.
     * @return the dimension value.
     */
    int getCartesianSpaceDim();
    /**
     * Gets the singular region area.
     * 
     * @return the eps.
     */
    double getEps();

    /**
     * Sets the singular region area.
     * 
     * @param eps
     *            The eps to set.
     */
    void setEps(double eps);

    /**
     * Gets the sample time.
     * 
     * @return the sample time.
     */
    public double getSampleTime();

    /**
     * Sets the sample time.
     * 
     * @param h
     *            The sample time to set.
     */
    public void setSampleTime(double h);

    /**
     * Gets the matrix of error position gain.
     * 
     * @return the diagonal matrix.
     */
    public DMatrix getKp();

    /**
     * Sets the matrix of error position gain.
     * The matrix must be of dimension 2 for planar robots.
     * @param kp
     *            The diagonal matrix to set.
     */
    public void setKp(DMatrix kp);

    /**
     * Gets the maximum damped factor.
     * 
     * @return the damped factor.
     */
    public double getLambdaMax();

    /**
     * Sets the maximum damped factor.
     * 
     * @param lambdaMax
     *            The damped factor to set.
     */
    public void setLambdaMax(double lambdaMax);

    /**
     * Gets the current position error.
     * 
     * @return the position error vector.
     */
    public RVector3d getPositionError();
    
    /**
     * Gets the current joints position.
     * @return the joints position vector.
     */
    public RVector getJointsPosition();
    
    /**
     * Gets the current joints velocity.
     * @return the joints velocity vector.
     */
    public RVector getJointsVelocity();  
    
    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and velocity.
     * 
     * @param pd
     *            the vector of desired end effector position.
     * @param ppd
     *            the vector of desired end effector velocity.
     */
    public void step(RVector pd, RVector ppd);
    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and velocity with velocity constraints.
     * 
     * @param pd
     *            the vector of desired end effector position.
     * @param ppd
     *            the vector of desired end effector velocity.
     * @param c
     *            the velocity constraint class.
     * @see ConstraintVelocity
     */
    public void step(RVector pd, RVector ppd, ConstraintVelocity c);

}
