/*
 * ClikJacobianInverse.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RMatrix3d;
import javax.robotics.vecmath.RMatrix4d;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;

import javax.vecmath.MismatchedSizeException;

/**
 * Closed Loop Inverse Kinematics with Jacobian inverse. The class implements
 * this scheme:<br>
 * <img src=doc-files/inversa.png alt="CLIK scheme with inverse" /> <br>
 * The inverse is computed using SVD decomposition:
 * <p>
 * J<sup>-1</sup> = J<sup>T</sup>*(J*J<sup>T</sup> + &lambda;<sup>2</sup>*I)<sup>-1</sup>
 * </br>where &lambda;<sup>2</sup> is the damped factor equals 1 - (&sigma;<sub>r</sub>/&epsilon;)*&lambda;<sup>2</sup><sub>max</sub>
 * if &sigma;<sub>r</sub>&lt; &epsilon;, equals 0 if &sigma;<sub>r</sub>&ge;
 * &epsilon;; &sigma;<sub>r</sub> is the smallest not null singular value.
 * </p>
 * <p>
 * The orientation error is computed as quaternion error.<br>
 * The integration method used is the forward Euler.
 * </p>
 * 
 * @author Carmine Lia
 * @since 1.0.2
 * @version 11/11/2005
 */
public class ClikJacobianInverse implements CLIK
{
    Robot robot;
    DMatrix Kp, Ko;
    double h = 0.001;
    double eps = 0.02;
    double lambdaMax = 0.02;
    int sizePos, sizeOr, dof;
    RVector q, qp, qpp, x, sum;// , qp_prev;
    RVector3d xo, orienErr, xp, posErr;
    RMatrix3d R = null;
    RVector3d p = null;
    Quaternion quat = null;
    RMatrix4d T = null;
    Matrix J, Ji, Jdot;

    /**
     * Costructor with default sample time, dt = 1e-3 and eps = 0.02,
     * lambdaMax=0.02.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param Ko
     *            the diagonal matrix of error orientationo gain.
     */
    public ClikJacobianInverse(Robot robot, DMatrix Kp, DMatrix Ko)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.Ko = Ko;
        init();
    }

    /**
     * Costructor with eps = 0.02, lambdaMax=0.02.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param Ko
     *            the diagonal matrix of error orientationo gain.
     * @param dt
     *            the sample time.
     */
    public ClikJacobianInverse(Robot robot, DMatrix Kp, DMatrix Ko, double dt)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.Ko = Ko;
        h = dt;
        init();
    }

    /**
     * Constructor.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param Ko
     *            the diagonal matrix of error orientationo gain.
     * @param dt
     *            the sample time.
     * @param eps
     *            the area of singular region.
     * @param lambdaMax
     *            the maximum of damped factor.
     */
    public ClikJacobianInverse(Robot robot, DMatrix Kp, DMatrix Ko, double dt,
            double eps, double lambdaMax)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.Ko = Ko;
        h = dt;
        this.eps = eps;
        this.lambdaMax = lambdaMax;
        init();

    }
    /**
     * Gets the singular region area.
     * 
     * @return the eps.
     */
    public final double getEps()
    {
        return eps;
    }

    /**
     * Sets the singular region area.
     * 
     * @param eps
     *            The eps to set.
     */
    public final void setEps(double eps)
    {
        this.eps = eps;
    }

    /**
     * Gets the sample time.
     * 
     * @return the sample time.
     */
    public final double getSampleTime()
    {
        return h;
    }

    /**
     * Sets the sample time.
     * 
     * @param h
     *            The sample time to set.
     */
    public final void setSampleTime(double h)
    {
        this.h = h;
    }

    /**
     * Gets the matrix of error position gain.
     * 
     * @return the diagonal matrix.
     */
    public final DMatrix getKp()
    {
        return Kp;
    }

    /**
     * Sets the matrix of error position gain.
     * 
     * @param kp
     *            The diagonal matrix to set.
     */
    public final void setKp(DMatrix kp)
    {
        Kp = kp;
    }

    /**
     * Gets the matrix of error orientation gain.
     * 
     * @return the diagonal matrix.
     */
    public final DMatrix getKo()
    {
        return Ko;
    }

    /**
     * Sets the matrix of error orientation gain.
     * 
     * @param ko
     *            The diagonal matrix to set.
     */
    public final void setKo(DMatrix ko)
    {
        Ko = ko;
    }

    /**
     * Gets the maximum damped factor.
     * 
     * @return the damped factor.
     */
    public final double getLambdaMax()
    {
        return lambdaMax;
    }

    /**
     * Sets the maximum damped factor.
     * 
     * @param lambdaMax
     *            The damped factor to set.
     */
    public final void setLambdaMax(double lambdaMax)
    {
        this.lambdaMax = lambdaMax;
    }

    /**
     * Gets the current position error.
     * 
     * @return the position error vector.
     */
    public final RVector3d getPositionError()
    {
        return posErr;
    }
    
        /**
     * Gets the current orientation error.
     * The error is computed form quaternion as:
     * <p>
     * &eta;(q)&epsilon<sub>d</sub> - &eta<sub>d</sub>&epsilon(q) -
     * S(&epsilon<sub>d</sub>)&epsilon(q)
     * </p> where &eta; and &epsilon; are the vector and the scalar components of quaternion respectively.
     * @return the orientation error vector.
     */
    public final RVector3d getOrientationError()
    {
        return orienErr;
    }
    
    /**
     * Gets the current joints position.
     * @return the joints position vector.
     */
    public final RVector getJointsPosition()
    {
        return q;
    }
    
    /**
     * Gets the current joints velocity.
     * @return the joints velocity vector.
     */
    public final RVector getJointsVelocity()
    {
        return qp;
    }
    /**
     * initialize CLIK variable
     */
    private final void init()
    {

        if(Ko.getSize() != 3)
            throw new MismatchedSizeException(
                    "The size of diagonal matrix Ko must be 3");
        if(Kp.getSize() != 3)
            throw new MismatchedSizeException(
                    "The size of diagonal matrix Kp must be 3");

        dof = robot.getDof();
        q = robot.getQ();
        qp = robot.getQp();
        // qp_prev = new RVector(qp);
        // sum = new RVector(dof);
        xo = new RVector3d();
        xp = new RVector3d();
        posErr = new RVector3d();
        orienErr = new RVector3d();
        T = new RMatrix4d();
        R = new RMatrix3d();
        p = new RVector3d();
    }

    /**
     * p-p<sub>d</sub>
     * 
     * @param p
     *            the current end effector position
     * @param pd
     *            the desired end effector position
     */
    final void endEffPositionError(RVector3d pd)
    {
        posErr.sub(pd, p);
    }

    /**
     * &eta;(q)&epsilon<sub>d</sub> - &eta<sub>d</sub>&epsilon(q) -
     * S(&epsilon<sub>d</sub>)&epsilon(q)
     * 
     * @param R
     *            the current end effector rotation matrix
     * @param quatd
     *            the quaternion desired
     * @param wd
     *            the angular velocity desired.
     */
    final void endEffQuaternionError(Quaternion quatd)
    {
        final Quaternion quat = new Quaternion();
        final RVector3d prod1 = new RVector3d();
        final RVector3d prod2 = new RVector3d();
        final RVector3d prod3 = new RVector3d();
        final RMatrix3d S = new RMatrix3d();
        
        quat.set(R);
        S.S(quat.getVector());
        // segno dei quaternioni
        if(quat.dot(quatd) < 0)
            quatd.negate();

        prod1.scale(quat.getScalar(), quatd.getVector());
        prod2.scale(quatd.getScalar(), quat.getVector());
        prod3.mul(S, quat.getVector());
        orienErr.sub(prod1, prod2);
        orienErr.add(prod3);
    }

    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and orientation. The joints velocity is computed as:<br>
     * <img src="doc-files/eqinversa.png" alt = "q'=inv(J)*(xd' +Ke)"/><br>
     * the joints position is computed by Euler integration:<br>
     * q(t+dt) = q(t) + q'(t)*dt;
     * 
     * @param pd
     *            the vector of desired end effector position.
     * @param ppd
     *            the vector of desired end effector velocity.
     * @param quatd
     *            the desired end effector quaternion.
     * @param wd
     *            the desired end effector angular velocity.
     */
    public void step(RVector3d pd, RVector3d ppd, Quaternion quatd,
            RVector3d wd)
    {
        // cinematica diretta
        robot.setQ(this.q);
        T = robot.kine();
        T.get(R, p);
        endEffPositionError(pd);
        endEffQuaternionError(quatd);

        xp.mul(Kp, posErr);
        xp.add(ppd);
        xo.mul(Ko, orienErr);
        xo.add(wd);
        x = new RVector(xp);
        x.append(xo);

        J = robot.jacobian();
        Ji = J.inverse(eps, lambdaMax);
        this.qp.mul(Ji, x);

        // trapezoidal integration
        // sum.add(this.qp, qp_prev);
        // this.q.scaleAdd(h * 0.5, sum, this.q);

        // euler integration
        this.q.scaleAdd(h, this.qp, this.q);

    }

}
