/*
 * ClikJacobianPosInverse.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.*;
import javax.vecmath.MismatchedSizeException;

/**
 * Closed Loop Inverse Kinematics with positional Jacobian inverse. The class
 * implements the same scheme depicted in
 * {@link javax.robotics.engine.clik.ClikJacobianInverse}: <br>
 * The inverse is computed using SVD decomposition:
 * <p>
 * J<sup>-1</sup> = J<sup>T</sup>*(J*J<sup>T</sup> + &lambda;<sup>2</sup>*I)<sup>-1</sup>
 * </br>where &lambda;<sup>2</sup> is the damped factor equals 1 - (&sigma;<sub>r</sub>/&epsilon;)*&lambda;<sup>2</sup><sub>max</sub>
 * if &sigma;<sub>r</sub>&lt; &epsilon;, equals 0 if &sigma;<sub>r</sub>&ge;
 * &epsilon;; &sigma;<sub>r</sub> is the smallest not null singular value.
 * </p>
 * <p>
 * The orientation error is not computed.<br>
 * The integration method used is the forward Euler.
 * </p>
 * 
 * @see #step(RVector, RVector) the solution without velocity
 *      constraint.
 * @see #step(RVector, RVector, ConstraintVelocity) the solution
 *      with velocity constraint.
 * @author Carmine Lia
 * @since 1.0.2
 * @version 11/11/2005
 */
public class ClikJacobianPosInverse implements CLIKR
{
    Robot robot;
    DMatrix Kp;
    double h = 0.001;
    double eps = 0.04;
    double lambdaMax = 0.04;
    int spaceDim, dof;
    RVector q, qp, qpp, q0Dot, sum, posErr, xp, p;
    RVector3d p_ = null;
    RMatrix4d T = null;
    Matrix J, Ji, Jp;
    
    /**
     * Costructor with default sample time, dt = 1e-3 and eps = 0.02,
     * lambdaMax=0.02, the space dimension = 3.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     */
    public ClikJacobianPosInverse(Robot robot, DMatrix Kp)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.spaceDim = 3;
        init();
    }
    
    /**
     * Costructor with default sample time, dt = 1e-3 and eps = 0.02,
     * lambdaMax=0.02.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param spaceDim
     *            the dimension of robot operational space.
     */
    public ClikJacobianPosInverse(Robot robot, DMatrix Kp, int spaceDim)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.spaceDim = spaceDim;
        init();
    }

    /**
     * Costructor with eps = 0.02, lambdaMax=0.02.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param spaceDim
     *            the dimension of robot operational space.
     * @param dt
     *            the sample time.
     */
    public ClikJacobianPosInverse(Robot robot, DMatrix Kp, int spaceDim,
            double dt)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.spaceDim = spaceDim;
        h = dt;
        init();
    }
    /**
     * Constructor.
     * 
     * @param robot
     *            the robot.
     * @param Kp
     *            the diagonal matrix of error position gain.
     * @param spaceDim
     *            the dimension of robot operational space.
     * @param dt
     *            the sample time.
     * @param eps
     *            the area of singular region.
     * @param lambdaMax
     *            the maximum of damped factor.
     */
    public ClikJacobianPosInverse(Robot robot, DMatrix Kp, double dt,
            int spaceDim,  double eps, double lambdaMax)
    {
        this.robot = robot;
        this.Kp = Kp;
        this.spaceDim = spaceDim;
        h = dt;
        this.eps = eps;
        this.lambdaMax = lambdaMax;
        init();

    }

    public final int getCartesianSpaceDim()
    {
        return spaceDim;
    }

    public final double getEps()
    {
        return eps;
    }

    public final void setEps(double eps)
    {
        this.eps = eps;
    }

    public final double getSampleTime()
    {
        return h;
    }

    public final void setSampleTime(double h)
    {
        this.h = h;
    }

    public final DMatrix getKp()
    {
        return Kp;
    }

    public final void setKp(DMatrix kp)
    {
        Kp = kp;
    }

    public final double getLambdaMax()
    {
        return lambdaMax;
    }

    public final void setLambdaMax(double lambdaMax)
    {
        this.lambdaMax = lambdaMax;
    }

    public final RVector3d getPositionError()
    {
        RVector3d returnValue = new RVector3d();
        switch (spaceDim)
        {
        case 1:
            returnValue.x = posErr.getElement(0);
            break;
        case 2:
            returnValue.x = posErr.getElement(0);
            returnValue.y = posErr.getElement(1);
            break;
        case 3:
            returnValue.x = posErr.getElement(0);
            returnValue.y = posErr.getElement(1);
            returnValue.z = posErr.getElement(2);
            break;
        default:
            returnValue = null;
            break;
        }
        return returnValue;
    }

    /**
     * Gets the current joints position.
     * @return the joints position vector.
     */
    public final RVector getJointsPosition()
    {
        return q;
    }
    
    /**
     * Gets the current joints velocity.
     * @return the joints velocity vector.
     */
    public final RVector getJointsVelocity()
    {
        return qp;
    }
    
    /**
     * initialize the clik variables.
     */
    private final void init()
    {
        if(spaceDim != Kp.getSize())
            new MismatchedSizeException("The size of diagonal matrix Kp must be "+ spaceDim);

        dof = robot.getDof();
        q = robot.getQ();
        qp = robot.getQp();
        // qp_prev = new RVector(qp);
        // sum = new RVector(dof);
        xp = new RVector(spaceDim);
        posErr = new RVector(spaceDim);
        T = new RMatrix4d();
        p_ = new RVector3d();
        p = new RVector(spaceDim);
        Jp = new Matrix(spaceDim, dof);
    }
    /**
     * p-p<sub>d</sub>
     * 
     * @param p
     *            the current end effector position
     * @param pd
     *            the desired end effector position
     */
    final void endEffPositionError(RVector pd)
    {
        posErr.sub(pd, p);
    }
    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and velocity. The joints velocity is computed as:<br>
     * <img src="doc-files/eqpinv.png" alt = "q'=pinv(J)*(xd' +Ke)"/><br>
     * the joints position is computed by Euler integration:<br>
     * q(t+dt) = q(t) + q'(t)*dt;
     * 
     * @param pd
     *            the RVector of desired end effector position.
     * @param ppd
     *            the RVector of desired end effector velocity.
     */
    public void step(RVector pd, RVector ppd)
    {
        // cinematica diretta
        robot.setQ(this.q);
        T = robot.kine();
        T.get(p_);
        p.set(p_);
        p.setSize(spaceDim);

        endEffPositionError(pd);

        xp.mul(Kp, posErr);
        xp.add(ppd);
        // calcolo dello Jacobiano posizionale
        J = robot.jacobian();
        J.getSubMatrix(0, spaceDim - 1, 0, dof - 1, Jp);
        // inversa a minimi quadrati smorzata (pseudo inversa smorzata)
        Ji = Jp.inverse(eps, lambdaMax);
        this.qp.mul(Ji, xp);

        // trapezoidal integration
        /*
         * sum.add(this.qp, qp_prev); this.q.scaleAdd(h*0.5, sum, this.q);
         */
        // euler integration
        this.q.scaleAdd(h, this.qp, this.q);

    }
    /**
     * Compute the joints position and velocity from a given desired end
     * effector position and velocity with velocity constraint. The joints
     * velocity is computed as:<br>
     * <img src="doc-files/eqvincolo.png" alt = "q'=pinv(J)*(xd'
     * +Ke)+(I-pinv(J)*J)*q0'"/><br>
     * the joints position is computed by Euler integration:<br>
     * q(t+dt) = q(t) + q'(t)*dt;
     * 
     * @param pd
     *            the RVector of desired end effector position.
     * @param ppd
     *            the RVector of desired end effector velocity.
     * @param constraint
     *            the velocity constraint class.
     * @see ConstraintVelocity
     */
    public void step(RVector pd, RVector ppd, ConstraintVelocity constraint)
    {
        final Matrix JiJp = new Matrix(dof);

        // cinematica diretta
        robot.setQ(this.q);
        T = robot.kine();
        T.get(p_);
        p.set(p_);
        p.setSize(spaceDim);

        endEffPositionError(pd);

        q0Dot = constraint.compute(this.q);

        xp.mul(Kp, posErr);
        xp.add(ppd);

        // calcolo dello Jacobiano posizionale
        J = robot.jacobian();
        J.getSubMatrix(0, spaceDim - 1, 0, dof - 1, Jp);
        // inversa a minimi quadrati smorzata (pseudo inversa smorzata)
        Ji = Jp.inverse(eps, lambdaMax);
        // pinv(J)*v
        this.qp.mul(Ji, xp);

        // calcolo di (I-pinv(J)*J)
        JiJp.mul(Ji, Jp);
        JiJp.sub(new Matrix(dof), JiJp);
        q0Dot.mul(JiJp, q0Dot);

        this.qp.add(q0Dot);
        // trapezoidal integration
        /*
         * sum.add(this.qp, qp_prev); this.q.scaleAdd(h*0.5, sum, this.q);
         */
        // euler integration
        this.q.scaleAdd(h, this.qp, this.q);

    }

}
