/*
 * ConstraintFunction.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.clik;

import javax.robotics.vecmath.RVector;

/**
 * Interface for the constraint function w(q).
 * <p>
 * <b>Example</b> <br>
 * define a constraint class:
 * 
 * <pre>
 * 
 *  public class MyFunctionConstraint extends ConstraintFunction
 *  {
 *  
 *      MyFunctionConstraint(double k0, ...)
 *      {
 *          super(k0);
 *          ...
 *      }
 *  
 *  
 *      // Here implement function {@code w(q)}
 *      public double function(RVector q)
 *      {
 *          ...
 *       }
 *  
 *  }
 *  
 * </pre>
 * 
 * use the class in your application:
 * 
 * <pre>
 * 
 *  public class MyApplication
 *  {
 *       CLIKR clik = new ClikPosJcobianInverse(...); 
 *       MyFunctionConstraint w = new MyFunctionConstraint(k0, ...);
 *       ...
 *       // pass the class to clik wich computes the velocity constraint as gradient of w(q) 
 *       clik.step(pd, pdd, q, qp, w);
 *  }
 *  
 * </pre>
 * 
 * </p>
 * 
 * @author Carmine Lia
 * @since 1.0.2
 */
public abstract class ConstraintFunction implements ConstraintVelocity
{
    private final double deltaQ = 1e-6;
    private double k0;

    public ConstraintFunction(double k0)
    {
        this.k0 = k0;
    }

    final public double getK0()
    {
        return k0;
    }

    final public void setK0(double k0)
    {
        this.k0 = k0;
    }

    /**
     * Interface for the constraint function {@code w(q)}.
     * 
     * @param q
     *            the joints position.
     * @return the function.
     */
    public abstract double function(RVector q);

    /**
     * Computes the constraint velocity as gradient of function {@code w(q)}.<br>
     * <img src="doc-files/vincolovel.png" alt="q0'=k0*grad(w(q))" />
     * 
     * @param q
     *            the joints position.
     * @return the current value of constraint.
     */
    final public RVector compute(RVector q)
    {
        final int n = q.getSize();
        final RVector gradient = new RVector(n);
        final RVector qDelta = new RVector(q);

        for (int i = 0; i < n; i++)
        {
            qDelta.set(q);
            qDelta.setElement(i, q.getElement(i) + deltaQ);
            gradient.setElement(i, (function(qDelta) - function(q)) / deltaQ);
        }
        gradient.scale(k0);
        return gradient;
    }
}
