/*
 * ComputedTorque.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.engine.JRoboOp;
import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.RVector;

/**
 * <p>
 * This class implements Computed Torque Method joints position controller. The
 * scheme of the control:<br>
 * <img src="doc-files/ctm.png" alt="scheme of computed torque method" /> <br>
 * the dynamics model of robot:
 * {@code B(q)*q'' + n(q,q') = u,  n(q,q') = C(q,q')*q'+ Fq'+ g(q)}
 * </p>
 * <p>
 * Wrapper class for C++ library ROBOOP.
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 7/10/2005
 * 
 */
public class ComputedTorque extends JRoboOp implements JointController
{
    private short robotID;

    private short controllerID = 0;
    private DMatrix Kp, Kd;

    /**
     * Computed Torque Method position controller class constructor.
     * 
     * @param robot
     *            Robot class.
     * @param Kp
     *            diagonal matrix joint position error gain matrix.
     * @param Kd
     *            diagonal matrix joint velocity error gain matrix.
     */
    public ComputedTorque(Robot robot, DMatrix Kp, DMatrix Kd)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        double[] Kp_ = new double[Kp.getSize()];
        double[] Kd_ = new double[Kd.getSize()];
        Kp.get(Kp_);
        Kd.get(Kd_);
        this.Kp = Kp;
        this.Kd = Kd;
        createComputedTorqueMethod(controllerID, robotID, Kp_, Kd_);
    }
    /**
     * Computed Torque Method position controller class constructor.
     * 
     * @param robot
     *            Robot class.
     * @param Kp
     *            diagonal matrix joint position error gain matrix.
     * @param Kd
     *            diagonal matrix joint velocity error gain matrix.
     */
    public ComputedTorque(Robot robot, double[] Kp, double[] Kd)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        this.Kp = new DMatrix(Kp);
        this.Kd = new DMatrix(Kd);
        createComputedTorqueMethod(controllerID, robotID, Kp, Kd);
    }
    /**
     * Gets Controller class ID.
     * 
     * @return ID
     */
    public final short getID()
    {
        return controllerID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.robotics.engine.Controller#getRobotID()
     */
    public final short getRobotID()
    {
        return robotID;
    }

    public final DMatrix getKp()
    {
        return this.Kp;
    }

    public final DMatrix getKd()
    {
        return this.Kd;
    }

    public short setKp(DMatrix Kp)
    {
        this.Kp = Kp;
        final double[] Kp_ = new double[Kp.getSize()];
        Kp.get(Kp_);
        return setKpCTM(controllerID, robotID, Kp_);
    }
    /**
     * Sets the position error gain matrix.
     * 
     * @param Kp
     *            position error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    public short setKp(double[] Kp)
    {
        this.Kp = new DMatrix(Kp);
        return setKpCTM(controllerID, robotID, Kp);
    }

    public short setKd(DMatrix Kd)
    {
        this.Kd = Kd;
        final double[] Kd_ = new double[Kd.getSize()];
        Kd.get(Kd_);
        return setKdCTM(controllerID, robotID, Kd_);
    }
    /**
     * Sets the velocity error gain matrix.
     * 
     * @param Kd
     *            velocity error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    public short setKd(double[] Kd)
    {
        this.Kd = new DMatrix(Kd);
        return setKdCTM(controllerID, robotID, Kd);

    }

    /**
     * Sets the output torque for a desired joint position and velocity.
     * 
     * @param qd
     *            array of desired joints position, joints velocity and joints
     *            acceleration.
     * @return torque.
     */
    public double[] torqueCmd(double[]... qd)
    {
        if(qd.length == 3)
            return torqueCmdCTM(controllerID, robotID, qd[0], qd[1], qd[2]);
        throw new RuntimeException(
                "The argument must be 3: desired joints position, desired joints velocity and desired joints acceleration");
    }
    /**
     * Sets the output torque for a desired joint position and velocity.
     * 
     * @param qd
     *            array of desired joints position, joints velocity and joints
     *            acceleration.
     * @return torque.
     */
    public RVector torqueCmd(RVector... qd)
    {

        if(qd.length == 3)
        {
            final double[] qd_ = new double[qd[0].getSize()];
            final double[] qpd_ = new double[qd[1].getSize()];
            final double[] qppd_ = new double[qd[2].getSize()];

            qd[0].get(qd_);
            qd[1].get(qpd_);
            qd[2].get(qppd_);
            return new RVector(torqueCmdCTM(controllerID, robotID, qd_, qpd_,
                    qppd_));
        }
        throw new RuntimeException(
                "The argument must be 3: desired joints position, desired joints velocity and desired joints acceleration");

    }
}
