/*
 * GravityCompensation.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.engine.JRoboOp;
import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;


/**
 * <p>
 * This class implements position controller with gravity compensation.  
 * The scheme of the control:<br>
 * <img src="doc-files/pdgso.png" alt="scheme of gravity compensation control" />
 * </p>
 * Wrapper class for C++ library ROBOOP.
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 7/10/2005
 */
public class GravityCompensation extends JRoboOp implements EndEffController
{
    private short robotID;
    
    private short controllerID;
    
    private int dof;
    
    /**
     * Gravity compensation controller class constructor.
     * 
     * @param robot Robot class.
     * @param Kdp end effector velocity gain
     * @param Kpp end effector position error gain
     * @param Kdo end effector orientation angular rate gain
     * @param Kpo end effector orientation error gain
     */
    public GravityCompensation(Robot robot, double Kdp, double Kpp, double Kdo, double Kpo)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        dof = robot.getDof();
        createGravityCompensation(controllerID, robotID, Kdp, Kpp, Kdo, Kpo);
    }
    /**
     * Gravity compensation controller class constructor with positional jacobian.
     * 
     * @param robot Robot class.
     * @param Kdp end effector velocity gain
     * @param Kpp end effector position error gain
     */
    public GravityCompensation(Robot robot, double Kdp, double Kpp)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        dof = robot.getDof();
        createGravityCompensation(controllerID, robotID, Kdp, Kpp, 0.0, 0.0);
    }
    /**
     * Gets Controller class ID.
     * @return ID
     */
    public final short getID()
    {
        return controllerID;
    }
   
 
    /* (non-Javadoc)
     * @see javax.robotics.engine.Controller#getRobotID()
     */
    public final short getRobotID()
    {
        return robotID;
    }   
   
    public RVector3d getPositionError()
    {
         return new RVector3d(getPosErrorGCOMP(controllerID));
    }
    
    public RVector3d getOrientationError()
    {
        return new RVector3d(getQuatErrorGCOMP(controllerID));
    }

    /**
     * Sets the output torque for a desired end effector position and angular position.
     * 
     * @param pd desired position
     * @param quatd desired vector angular position: quaternion (s,v1,v2,v3)
     * @return the output torque
     */
    public double[] torqueCmd(double[] pd, double[] quatd)
    {
       return  torqueCmdGCOMP(controllerID, robotID, pd, quatd, (short)dof);
    }
    /**
     * Sets the output torque for a desired end effector position and angular position.
     * 
     * @param pd desired position
     * @param quatd desired vector angular position: quaternion (s,v1,v2,v3)
     * @return the output torque
     */
   public RVector torqueCmd(RVector3d pd, Quaternion quatd)
    {
       final double[] pd_ = new double[3];
       final double[] quatd_ = new double[4];
       
       pd.get(pd_);
       quatd_[0] = quatd.w;
       quatd_[1] = quatd.x;
       quatd_[2] = quatd.y;
       quatd_[3] = quatd.z;
       
       return  new RVector(torqueCmdGCOMP(controllerID, robotID, pd_, quatd_, (short)dof));
    }
    
    /**
     * Sets the output torque for a desired end effector position.
     * @param pd desired position
     * @return the output torque
     */
    public RVector torqueCmd(RVector3d pd)
    {
        final double[] pd_ = new double[3];
       
       pd.get(pd_);
       
       return  new RVector(torqueCmdGCOMP(controllerID, robotID, pd_));
    }
    /**
     * Sets the output torque for a desired end effector position.
     * @param pd desired position
     * @return the output torque
     */
    public RVector torqueCmd(RVector pd)
    {
       final int n = pd.getSize();
       final double[] pd_ = new double[n];
       
       pd.get(pd_);
       
       return  new RVector(torqueCmdGCOMP(controllerID, robotID, pd_));
    }
    
    /**
     * Sets the output torque for a desired end effector position and angular position.
     * The function call @link #torqueCmd(RVector3d, Quaternion)
     * @param pdpp null vector
     * @param pdp null vector
     * @param pd desired position
     * @param wdp null vector
     * @param quatd desired vector angular position: quaternion (s,v1,v2,v3)
     * @return the output torque
     */
    public RVector torqueCmd(RVector3d pdpp, RVector3d pdp, RVector3d pd, RVector3d wdp, RVector3d wd, Quaternion quatd)
    {
        return this.torqueCmd(pd, quatd);
    }
    
    /**
     * Sets the output torque for a desired end effector position.
     * The function call @link #torqueCmd(RVector3d).
     * @param pdpp null vector
     * @param pdp  null vector
     * @param pd   desired position
     * @return the output torque
     */
    public RVector torqueCmd(RVector3d pdpp, RVector3d pdp, RVector3d pd)
    {
        return this.torqueCmd(pd);
    }
   
}