/*
 * ImpedanceControl.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.engine.JRoboOp;
import javax.robotics.engine.robots.Robot;

/**
 * <p>
 * This class implements Impedance position controller. <b>(NOT IMPLEMENTED)</b>
 * </p>
 * Wrapper class for C++ library ROBOOP.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * 
 */
public class ImpedanceControl extends JRoboOp // implements Controller
{
    // robot id
    private short robotID;

    private short controllerID;

    /**
     * Impedance controller class constructor.
     * <p>
     * Call C++ constructor
     * 
     * <pre>
     * Impedance(const Robot_basic &amp;robot, 
     *                             const DiagonalMatrix &amp;Mp, const DiagonalMatrix &amp;Dp,
     *                             const DiagonalMatrix &amp;Kp, const Matrix &amp;Km,
     *                             const DiagonalMatrix &amp;Mo, const DiagonalMatrix &amp;Do,
     *                             const DiagonalMatrix &amp;Do)
     * </pre>
     * 
     * </p>
     * 
     * @param robot
     *            Robot class.
     * @param Mp
     *            diagonal matrix, translational impedance inertia matrix
     * @param Dp
     *            diagonal matrix, translational impedance damping matrix
     * @param Kp
     *            diagonal matrix, translational impedance stiffness matrix
     * @param Km
     *            matrix
     * @param Mo
     *            diagonal matrix, rotational impedance inertia matrix
     * @param Do
     *            diagonal matrix, rotational impedance damping matrix
     * @param Ko
     *            diagonal matrix, rotational impedance stiffness matrix
     */
    public ImpedanceControl(Robot robot, double[] Mp, double[] Dp, double[] Kp,
            double[] Km, double[] Mo, double[] Do, double[] Ko)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        createImpedance(controllerID, robotID, Mp, Dp, Kp, Km, Mo, Do, Ko);
    }

    /**
     * Gets Controller class ID.
     * 
     * @return ID
     */
    final public short getID()
    {
        return controllerID;
    }

    /**
     * Gets converge flag.
     * 
     * @return converge flag
     */
    final public boolean getConverge()
    {
        return getConverge(controllerID);
    }

    /**
     * Gets error flag.
     * 
     * @return error flag.
     */
    public short getError()
    {
        return getError(controllerID);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.robotics.engine.Controller#getRobotID()
     */
    public short getRobotID()
    {
        return robotID;
    }

    /**
     * Sets the compliant trajectory for a desired trajectory effector:
     * acceleration, velocity, position, angular acceleration, angular velocity,
     * angular position.
     * <p>
     * Call C++ Impedance method
     * 
     * <pre>
     * control(Robot_basic &amp;robot, 
     *                                const ColumnVector &amp;pdpp const ColumnVector &amp;pdp,
     *                                const ColumnVector &amp;pd, const ColumnVector &amp;wdp,
     *                                const ColumnVector &amp;wd, const Quaternion &amp;qd,
     *                                const ColumnVector &amp;f, const Columnvector &amp;n, 
     *                                const Real dt)
     * </pre>
     * 
     * </p>
     * 
     * @param pdpp
     *            desired acceleration
     * @param pdp
     *            desired velocity
     * @param pd
     *            desired position
     * @param wdp
     *            desired angular acceleration
     * @param wd
     *            desired angular velocity
     * @param qd
     *            desired vector angular position: quaternion (s,v1,v2,v3)
     * @param f
     *            end effector contact force
     * @param n
     *            end effector contact moment
     * @param dt
     *            sample time
     * @return 0 successfull otherwise indicates the wrong size of input
     *         vectors.
     */
    public short control(double[] pdpp, double[] pdp, double[] pd,
            double[] wdp, double[] wd, double[] qd, double[] f, double[] n,
            double dt)
    {
        return impedanceControl(controllerID, robotID, pdpp, pdp, pd, wdp, wd,
                qd, f, n, dt);
    }

}
