/*
 * JointController.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.RVector;


/**
 * <p>
 * Interface for different schemes of robot joints position controller.  
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 7/10/2005
 */
public interface JointController extends Controller
{
    /**
     * Incorrect size of error gain matrix
     */
    static final short WRONG_SIZE = -1;
    
    /**
     * Gets the position error gain matrix.
     * @return position error gain diagonal matrix
     */
    DMatrix getKp();
    /**
     * Gets the velocity error gain matrix.
     * @return velocity error gain diagonal matrix
     */
    DMatrix getKd();
    
    /**
     * Sets the position error gain matrix.
     * <p>
     * Call C++ method <pre>set_Kp(const DiagonalMatrixr &Kp)</pre>
     * </p>
     * @param Kp position error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    short setKp(DMatrix Kp);
    
    /**
     * Sets the velocity error gain matrix.
     * <p>
     * Call C++ method <pre>set_Kd(const DiagonalMatrixr &Kd)</pre>
     * </p>
     * @param Kd velocity error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    short setKd(DMatrix Kd);
    
    
    /**
     * Sets the output torque for a desired joint position and velocity and acceleration.
     * <p>
     * Call C++  method <pre>torque_cmd(Robot_basic &robot, const ColumnVector &qd, const ColumnVector &qpd)
     * or torque_cmd(Robot_basic &robot, const ColumnVector &qd, const ColumnVector &qpd, const ColumnVector &qppd)
      * </pre>
     * </p>
     * 
     * @param qd array of the vectors desired joints position, velocity, acceleration.
     * @return torque.
     */
    public double[] torqueCmd(double[]... qd);
    /**
     * Sets the output torque for a desired joint position and velocity and acceleration.
     * <p>
     * Call C++  method <pre>torque_cmd(Robot_basic &robot, const ColumnVector &qd, const ColumnVector &qpd)
     * or torque_cmd(Robot_basic &robot, const ColumnVector &qd, const ColumnVector &qpd, const ColumnVector &qppd)
      * </pre>
     * </p>
     * 
     * @param qd array of the vectors desired joints position, velocity, acceleration.
     * @return torque.
     */
    public RVector torqueCmd(RVector... qd);
}    
    
    