/*
 * RobustControl.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.engine.controllers;

import javax.robotics.engine.JRoboOp;
import javax.robotics.engine.robots.Robot;
import javax.robotics.vecmath.DMatrix;
import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.RVector;

/**
 * <p>
 * This class implements the Robust Control of joints position controller.
 * The scheme of the control:<br>
 * <img src="doc-files/robust.png" alt="scheme of robust control" />
 * <br>
 * the dynamics model of robot: 
 * {@code B(q)*q'' + n(q,q') = u,  n(q,q') = C(q,q')*q'+ Fq'+ g(q)} 
 * </p>
 * the robustness {@code w} is computed as:<br>
 * <pre>
 * w = &rho;/z.vers() for |z.norm()| &ge; &epsilon;
 * w = (&rho;/&epsilon;)*z for |z.norm()| &lt; &epsilon;
 * </pre> 
 * <code>D</code> is a 2*dof-by-dof matrix <code>D= [O; I]</code><br>
 * <code>Q</code> is the solution of Lyapunov equation: <code>H<sup>T</sup>*Q + Q*H = -I</code> where <code>H</code> is a 2*dof-by-2*dof matrix <code>H = [O, I; -Kp, -Kd]</code><br>
 * the inertia matrix <code>B</code> is constant;<br>
 * the non linear compensation {@code n(q,q') = Fv*q' + g(q)} (viscous friction + gravity torques)  
 * <p>
 * Wrapper class for C++ library ROBOOP.
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.4
 * 
 */
public class RobustControl extends JRoboOp implements JointController
{
    private short robotID;

    private short controllerID = 0;
    private DMatrix Kp, Kd;
    /**
     * Robust control position controller class constructor.
     * The constructor computes the solution of Lyapunov equation by inner function {@link Matrix#lyap(Matrix)} but it could fail.
     * @param robot
     *            Robot class.
     * @param Kp
     *            diagonal matrix joint position error gain matrix.
     * @param Kd
     *            diagonal matrix joint velocity error gain matrix.
     * @param B   a constant robot inertia matrix.
     * @param rho the robustness gain.
     * @param eps the chattering compensation.
     * @see Matrix#lyap(Matrix)
     */
    public RobustControl(Robot robot, DMatrix Kp, DMatrix Kd, Matrix B, double rho, double eps)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        int dof = robot.getDof();
        this.Kp = Kp;
        this.Kd = Kd;
        final double[] Kp_ = new double[Kp.getSize()];
        final double[] Kd_ = new double[Kd.getSize()];
        final double[] B_ = new double[B.getNumRow()*B.getNumCol()];
        final double[] Q_ = new double[4*dof*dof];
        final Matrix H = new Matrix(2*dof);
        final Matrix P = new Matrix(2*dof);
        
        Kp.get(Kp_);
        Kd.get(Kd_);
        B.get(B_);
        // solution of Lyapunov equation
        H.setZero();
        for(int i=0; i < dof; i++)
        {
            H.setElement(i, dof+i, 1.0);
            H.setElement(dof+i, i, -Kp_[i]);
            H.setElement(dof+i, dof+i, -Kd_[i]);
            
        }
        H.transpose();
        H.lyap(P).get(Q_);
  /*       
        System.out.println("H\n" + H);
        System.out.println("Q\n" + Q);
        System.out.println("P\n" + P);
        System.out.println("B\n" + B);
  
        Matrix P1 = new Matrix(4);
        Matrix P2 = new Matrix(4);
        P2.mulTransposeRight(Q, H);
        P1.mul(H, Q);
        P1.add(P2);
        P1.printf();
    */   
        createRobustControl(controllerID, robotID, Kp_, Kd_, B_, Q_, rho, eps);
    }
    /**
     * Robust control position controller class constructor with a pre-computed solution of Lyapunov equation.
     * @param robot
     *            Robot class.
     * @param Kp
     *            diagonal matrix joint position error gain matrix.
     * @param Kd
     *            diagonal matrix joint velocity error gain matrix.
     * @param B   a constant robot inertia matrix.
     * @param Q   the solution of Lyapunov equation.
     * @param rho the robustness gain.
     * @param eps the chattering compensation.
     * @see Matrix#lyap(Matrix)
     */
    public RobustControl(Robot robot, DMatrix Kp, DMatrix Kd, Matrix B, Matrix Q, double rho, double eps)
    {
        controllerID = JRoboOp.ID;
        robotID = robot.getID();
        int dof = robot.getDof();
        this.Kp = Kp;
        this.Kd = Kd;
        final double[] Kp_ = new double[Kp.getSize()];
        final double[] Kd_ = new double[Kd.getSize()];
        final double[] B_ = new double[B.getNumRow()*B.getNumCol()];
        final double[] Q_ = new double[4*dof*dof];
        Kp.get(Kp_);
        Kd.get(Kd_);
        B.get(B_);
        Q.get(Q_);
        createRobustControl(controllerID, robotID, Kp_, Kd_, B_, Q_, rho, eps);
    }
      
    /**
     * Gets Controller class ID.
     * 
     * @return ID
     */
    public final short getID()
    {
        return controllerID;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.robotics.engine.Controller#getRobotID()
     */
    public final short getRobotID()
    {
        return robotID;
    }

    public final DMatrix getKp()
    {
        return this.Kp;
    }

    public final DMatrix getKd()
    {
        return this.Kd;
    }

    public short setKp(DMatrix Kp)
    {
        this.Kp = Kp;
        final double[] Kp_ = new double[Kp.getSize()];
        Kp.get(Kp_);
        return setKpRC(controllerID, robotID, Kp_);
    }
    /**
     * Sets the position error gain matrix.
     * @param Kp position error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    public short setKp(double[] Kp)
    {
        this.Kp = new DMatrix(Kp);
        return setKpCTM(controllerID, robotID, Kp);
    }
   
    public short setKd(DMatrix Kd)
    {
        this.Kd = Kd;
        final double[] Kd_ = new double[Kd.getSize()];
        Kd.get(Kd_);
        return setKdRC(controllerID, robotID, Kd_);
    }
    /**
     * Sets the velocity error gain matrix.
     * @param Kd velocity error gain diagonal matrix
     * @return 0 or WRONG_SIZE
     */
    public short setKd(double[] Kd)
    {
        this.Kd = new DMatrix(Kd);
        return setKdRC(controllerID, robotID, Kd);

    }

    /**
     * Sets the output torque for a desired joint position and velocity.
     * @param qd
     *            array of desired joints position, joints velocity and joints
     *            acceleration.
     * @return torque.
     */
    public double[] torqueCmd(double[]... qd)
    {
        if(qd.length == 3)
            return torqueCmdRC(controllerID, robotID, qd[0], qd[1], qd[2]);
        throw new RuntimeException(
                    "The arguments must be 3: desired joints position, desired joints velocity and desired joints acceleration");
    }
    /**
     * Sets the output torque for a desired joint position and velocity.
     * 
     * @param qd
     *            array of desired joints position, joints velocity and joints
     *            acceleration.
     * @return torque.
     */
    public RVector torqueCmd(RVector... qd)
    {

        if(qd.length == 3)
        {
            final double[] qd_ = new double[qd[0].getSize()];
            final double[] qpd_ = new double[qd[1].getSize()];
            final double[] qppd_ = new double[qd[2].getSize()];

            qd[0].get(qd_);
            qd[1].get(qpd_);
            qd[2].get(qppd_);
            return new RVector(torqueCmdRC(controllerID, robotID, qd_, qpd_,
                    qppd_));
        }
        throw new RuntimeException(
                    "The arguments must be 3: desired joints position, desired joints velocity and desired joints acceleration");

    }
}
