/*
 * Euler2.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.ode;

import javax.robotics.vecmath.RVector;
import javax.vecmath.MismatchedSizeException;



/**
 * <p>
 * This class implements forward Euler integration method for systems of two first order differential equations.
 * <p><code>x''=f(x, x') with x(0)=x<sub>0</sub>, x'(0)= y<sub>0</sub>, is equivalent to system of two first order of differential equation:</br>
 * y' = f(x, y)</br>
 * x' = y</br>
 * with x(0)=x<sub>0</sub>, y(0)= y<sub>0</sub>
 * </code></p>  
 * </p>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.3
 * @version 09/11/2005
 * 
 */
public class Euler2  implements DerivableFunction2, Solver2
{
    double h;
    double time;
    RVector lastValuePosition, lastValueVelocity;
    
    private int size;
    private DerivableFunction2 target;
    private RVector initialConditionPosition, initialConditionVelocity;
   
    /**
     * Defines numerical integration for x'' = f(x, x') as system of two first order differential equation.
     * @param dt time interval of integration [sec]
     * @param initialConditionPosition position at time zero x(0).
     * @param initialConditionVelocity velocity at time zero x'(0).
     */
    public Euler2(double dt, RVector initialConditionPosition, RVector initialConditionVelocity)
    {
        this(dt, initialConditionPosition, initialConditionVelocity, null);
    }
    
    /**
     * Defines numerical integration for x'' = f(x, x') as system of two first order differential equation.
     * @param dt time interval of integration [sec]
     * @param initialConditionPosition position at time zero x(0).
     * @param initialConditionVelocity velocity at time zero x'(0).
     * @param target differential equation to be integrated
     */
    public Euler2(double dt, RVector initialConditionPosition, RVector initialConditionVelocity, DerivableFunction2 target)
    {
        size = initialConditionPosition.getSize();
        if(size != initialConditionVelocity.getSize())
            throw new MismatchedSizeException("The size of the position and velocity vectors must be equals");
 
        this.h = dt;
        lastValuePosition = new RVector(initialConditionPosition);
        lastValueVelocity = new RVector(initialConditionVelocity);
        this.initialConditionPosition = initialConditionPosition;
        this.initialConditionVelocity = initialConditionVelocity;
        this.target = target;
        time = startTime;
    }
    
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#setSampleTime(double)
     */
    public final void setSampleTime(double dt)
    {
        this.h = dt;
    }
    
    public final double getSampleTime()
    {
       return this.h;
    }
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#setInitialCondition(RVector)
     */
    public final void setInitialConditionPosition(RVector initialCondition)
    {
        lastValuePosition = new RVector(initialCondition);
        initialConditionPosition = initialCondition;
    }
    
    public final void setInitialConditionVelocity(RVector initialCondition)
    {
        lastValueVelocity = new RVector(initialCondition);
        initialConditionVelocity = initialCondition;
    }
    
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#getInitialCondition()
     */
    public final RVector getInitialConditionPosition()
    {
        return initialConditionPosition;
    }
    
    public final RVector getInitialConditionVelocity()
    {
        return initialConditionVelocity;
    }
   
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#getLastValue()
     */
    public final RVector getLastValuePosition()
    {
        return lastValuePosition;
    }
   
    public final RVector getLastValueVelocity()
    {
        return lastValueVelocity;
    }
   
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#getTime()
     */
    public final double getTime()
    {
        return time;
    }
    
    public final void resetTime()
    {
        time = 0.0;
    }
    
    public void setTargetFunction(DerivableFunction2 target)
    {
        this.target = target;
        
    }
    /* (non-Javadoc)
     * @see javax.robotics.engine.Solver#step(RVector)
     */
    public void step(RVector position, RVector velocity)
    {
      final RVector xdot = xDot2(position, velocity);
      
      lastValuePosition.scaleAdd(h, lastValueVelocity, lastValuePosition);  
      lastValueVelocity.scaleAdd(h, xdot, lastValueVelocity);
            
      time += h;
    }
    
       
    /* (non-Javadoc)
     * @see javax.robotics.engine.DerivableFunction#xDot(double[])
     */
    public RVector xDot2(RVector x, RVector y)
    {
        final RVector temp;
        temp = (target != null)? target.xDot2(x, y):null;
        /*if(target != null)
        {
            return target.xDot2(x, y);
        }
        */
        return temp;
    }

    
    
}
