/*
 * Solver.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.ode;

import javax.robotics.vecmath.RVector;



/**
 * <p>
 * This class is an interface for differents numerical integration methods of first order differential equations.</br>  
 * The implementation of a solver could be done writing:
 * <pre>
 * ...
 * public MyODESolver implements DerivableFunction
 * {
 *      ...
 *      RVector x0 = new RVector(n);
 *      // Creating the solver
 *      Solver mySolver = Euler(0.1, // time interval integration
 *                              x0,  // initial condition 
 *                              this // function xDot is passed to Solver
 *                             );
 *      ...
 *      
 *      // integration at time t+dt of x' = f(x) implemented by xDot(x) function
 *      x_dt = mySolver.step(x_t);
 *      
 *      // current time integration
 *      t_dt = mySolver.getTime();
 *      
 *      // current integrated value
 *      x_dt = mySolver.getLastValue();
 *      
 *      ...
 *      
 *      // implementation of x' = f(x)
 *      public RVector xDot(RVector x)
 *      {
 *          ...
 *          // code to calculate x' = f(x)
 *          ...
 *          return xdot;
 *      }
 * }
 * </pre> 
 * </p>
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.0
 * @version 09/11/2005
 * 
 */
public interface Solver
{
    double startTime = 0.0;
    
    /**
     * Sets the interval of integration.
     * @param dt interval [sec]
     */
    public void setSampleTime(double dt);
    /**
     * Gets the interval of integration.
     * @return the dt interval [sec]
     */
    public double getSampleTime();
    /**
     * Gets current time of integration.
     * @return the current time [sec]
     */
    public double getTime();
    /**
     * Sets the initial condition for position x(0) = x<sub>0</sub>.
     * @param initialCondition
     */
    public void setInitialCondition(RVector initialCondition);
   
    /**
     * Gets the initial condition for position x(0).
     * @return the initialCondition
     */
    public RVector getInitialCondition();
   
    /**
     * Gets the current value of position x computed by <code>step</code> function.
     * @return the current position.
     */
    public RVector getLastValue();
    
    /**
     * Computes step integration from time t to t+dt of x' = f(x).
     * The params last value position is modified.
     * @param x argument of f(x)
     */
    public void step(RVector x);
    
}
