/*
 * MultiPlot.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.plot;

import java.awt.Color;

import javax.robotics.vecmath.RVector3d;
import javax.robotics.vecmath.RVector;

import ptolemy.plot.Plot;

/**
 * A flexible signal plotter. The plot can be configured and data can be provided either through a file with commands or through direct invocation of the public methods of the class.
 * This class extends vector plotting function of MultiPlot class by <a href="http://ptolemy.eecs.berkeley.edu">Ptolemy</a>
 
    H or ?: Display a simple help message.</br> 
    Cntrl-D or Q: quit</br>
    
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.2
 * 
 */
public class MultiPlot extends Plot
{
    /**
     * 
     */
    private static final long serialVersionUID = 3977578091449430832L;

    public MultiPlot()
    {
        super();
    }
    
    /**
     * Constructor of plot graphics.
     * @param title the title.
     * @param xLabel the label of x axis.
     * @param yLabel the label of y axis.
     * @param legend the legend.
     */
    public MultiPlot(String title, String xLabel, String yLabel, String... legend)
    {
        super();
      //  this.setSize(350, 250); 
        this.setTitle(title);
        this.setXLabel(xLabel);
        this.setYLabel(yLabel);
        this.setButtons(true);
        this.setBackground(Color.WHITE);
        for(int i=0; i<legend.length; i++)
            this.addLegend(i, legend[i]);
    }
    /**
     * Adds vector to the plot.
     * The third argument indicates whether the points should be connected by a line to the previous point. 
     * @param x The X position of the new points.
     * @param y The vector of Y positions of the new points.
     * @param connected If true, a line is drawn to connect to the previous point.
     */
    public void addVector(double x, RVector y, boolean connected)
    {
         final int size = y.getSize();
         for(int i=0; i< size; i++)
            this.addPoint(i, x, y.getElement(i), connected);
    }
    /**
     * Adds vector to the plot.
     * The third argument indicates whether the points should be connected by a line to the previous point. 
     * @param x The X position of the new points.
     * @param y The vector of Y positions of the new points.
     * @param connected If true, a line is drawn to connect to the previous point.
     */
    public void addVector(double x, RVector3d y, boolean connected)
    {
            this.addPoint(0, x, y.x, connected);
            this.addPoint(1, x, y.y, connected);
            this.addPoint(2, x, y.z, connected);
    }
    /**
     * Adds vector to the plot from specified dataset.
     * The third argument indicates whether the points should be connected by a line to the previous point.
     * @param dataSet the starting dataset. 
     * @param x The X position of the new points.
     * @param y The vector of Y positions of the new points.
     * @param connected If true, a line is drawn to connect to the previous point.
     */
    public void addVector(int dataSet, double x, RVector y, boolean connected)
    {
        
        int n = y.getSize()+dataSet;
        int k=0;
        for(int i=dataSet; i<n; i++)
            this.addPoint(i, x, y.getElement(k++), connected);
    }
    /**
     * Adds vector to the plot from specified dataset.
     * The third argument indicates whether the points should be connected by a line to the previous point.
     * @param dataSet the starting dataset. 
     * @param x The X position of the new points.
     * @param y The vector of Y positions of the new points.
     * @param connected If true, a line is drawn to connect to the previous point.
     */
    public void addVector(int dataSet, double x, RVector3d y, boolean connected)
    {
            this.addPoint(dataSet,   x, y.x, connected);
            this.addPoint(dataSet+1, x, y.y, connected);
            this.addPoint(dataSet+2, x, y.z, connected);
    }

}
