/*
 * MultiPlotFrame.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.plot;

import java.awt.Color;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.robotics.image.ImageFileChooser;
import javax.robotics.image.ScreenImage;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;

import ptolemy.plot.PlotBox;

/**
 * Class to display multiple plot in a window. Example.
 * 
 * <pre>
 *   {@code
 *   
 *   ...
 *   MultiPlotFrame mpf = new MultiPlotFrame();
 *   MultiPlot mp1 = new MultiPlot();
 *   MultiPlot mp2 = new MultiPlot();
 *   MultiPlot mp3 = new MultiPlot();
 *   MultiPlot mp4 = new MultiPlot();
 *   mpf.add(mp1);
 *   mpf.add(mp2);
 *   mpf.add(mp3);
 *   mpf.add(mp4);
 *   mpf.create();
 *   ...
 *   
 *   }
 * </pre>
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 01/11/2005
 */
public class MultiPlotFrame extends JFrame implements ActionListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 3257845493651682612L;

    private final String title;
    private final MultiPlotPanel plotsPanel;
    private final JMenuBar menuBar = new JMenuBar();
    // file menu
    private final JMenuItem printItem = new JMenuItem("Print");
    private final JMenuItem saveItem = new JMenuItem("Save");
    private final JMenuItem closeItem = new JMenuItem("Close");
    private final JMenuItem exitItem = new JMenuItem("Exit");
    // special menu
    private final JMenuItem resetItem = new JMenuItem("Reset axes");
    private final JMenuItem fillItem = new JMenuItem("Fill plots");
    private final JMenuItem showItem = new JMenuItem("Show buttons");
    private final JMenuItem hideItem = new JMenuItem("Hide buttons");
    // help menu
    private final JMenuItem keyItem = new JMenuItem("Key bindings");
    private final JMenuItem aboutItem = new JMenuItem("About");

    /**
     * Default constructor of a 2x2 grid of plots.
     * 
     * @throws HeadlessException
     */
    public MultiPlotFrame() throws HeadlessException
    {
        this("Plot Window", 2, 2);
    }

    /**
     * Constructor of a 2x2 grid of plots.
     * 
     * @param title
     *            the window title.
     * @throws HeadlessException
     */
    public MultiPlotFrame(String title) throws HeadlessException
    {
        this(title, 2, 2);
    }

    /**
     * Constructor of a plots grid with the specified number of rows and
     * columns.
     * 
     * @param title
     *            the window title.
     * @param nRows
     *            the number of rows.
     * @param nCols
     *            the number of columns.
     * @throws HeadlessException
     */
    public MultiPlotFrame(String title, int nRows, int nCols)
            throws HeadlessException
    {
        super(title);
        this.title = title;
        // this.setPreferredSize(new Dimension(800, 600));
        this.setBackground(Color.WHITE);
        plotsPanel = new MultiPlotPanel(title, nRows, nCols);
        createMenuBar();
    }

    /**
     * Adds a new plot to the window.
     * 
     * @param mp
     *            the plot.
     */
    public void add(MultiPlot mp)
    {
        /*
         * if(plots.size() > maxPlots) throw new RuntimeException("The maximum
         * number of plots must be " + maxPlots);
         */
        plotsPanel.add(mp);
    }

    /**
     * Creates and displays the window.
     */
    public void create()
    {

        getContentPane().add(plotsPanel);
        setJMenuBar(menuBar);
        pack();
        setVisible(true);
    }

    private void createMenuBar()
    {
        final JMenu fileMenu = new JMenu("File");
        final JMenu specialMenu = new JMenu("Special");
        final JMenu helpMenu = new JMenu("Help");
        saveItem.addActionListener(this);
        printItem.addActionListener(this);
        closeItem.addActionListener(this);
        exitItem.addActionListener(this);

        resetItem.addActionListener(this);
        fillItem.addActionListener(this);
        showItem.addActionListener(this);
        hideItem.addActionListener(this);

        keyItem.addActionListener(this);
        aboutItem.addActionListener(this);

        // file menu
        fileMenu.add(saveItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(printItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(closeItem);
        fileMenu.add(exitItem);

        // special menu
        specialMenu.add(resetItem);
        specialMenu.add(new JSeparator());
        specialMenu.add(fillItem);
        specialMenu.add(new JSeparator());
        specialMenu.add(showItem);
        specialMenu.add(hideItem);

        // help menu
        helpMenu.add(keyItem);
        helpMenu.add(new JSeparator());
        helpMenu.add(aboutItem);

        menuBar.add(fileMenu);
        menuBar.add(specialMenu);
        menuBar.add(helpMenu);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent event)
    {
        final Object target = event.getSource();

        if(target.equals(printItem))
        {
            plotsPanel.showButtons(false);

            final PrinterJob printJob = PrinterJob.getPrinterJob();

            PageFormat pageFormat = printJob.defaultPage();
            pageFormat = printJob.validatePage(pageFormat);
            printJob.setPrintable(plotsPanel, pageFormat);
            if(printJob.printDialog())
            {
                try
                {
                    printJob.print();
                }
                catch (Exception ex)
                {
                    JOptionPane.showMessageDialog(this, "Printing failed:\n"
                            + ex.toString(), "Print Error",
                            JOptionPane.WARNING_MESSAGE);

                }
            }
            plotsPanel.showButtons(true);
            return;
        }

        if(target.equals(saveItem))
        {
            try
            {
                plotsPanel.showButtons(false);

                final ImageFileChooser fc = new ImageFileChooser();
                final String fileName = fc.setFileName(title + ".png");
                final ScreenImage plots = new ScreenImage();
                ImageIO.write(plots.createImage(plotsPanel), "png", new File(
                        fileName));
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }

            plotsPanel.showButtons(true);
            return;
        }

        if(target.equals(closeItem))
        {
            this.removeAll();
            this.setVisible(false);

        }
        if(target.equals(exitItem))
        {
            this.removeAll();
            this.setVisible(false);
            System.exit(0);

        }
        if(target.equals(fillItem))
        {
            plotsPanel.fillPlots();
        }

        if(target.equals(resetItem))
        {
            plotsPanel.resetPlots();
        }

        if(target.equals(showItem))
        {
            plotsPanel.showButtons(true);
        }

        if(target.equals(hideItem))
        {
            plotsPanel.showButtons(false);
        }

        if(target.equals(keyItem))
        {
            JOptionPane
                    .showMessageDialog(
                            this,
                            "  Left mouse button: Zooming\n"

                                    + "   Cntrl-c:  copy the plot as image\n"
                                    + "   Cntrl-p:  show the printer dialog\n"
                                    + "   D: dump plot data to standard out (XML Format)\n"
                                    + "   E: export plot to standard out (EPS format)\n"
                                    + "   F: fill plot\n"
                                    + "   H or ?: print help message (this message)\n"
                                    + "   Cntrl-D or Q: quit\n"
                                    + "For more information, see\n"
                                    + "http://ptolemy.eecs.berkeley.edu/java/ptplot\n",

                            "Ptolemy Plot Help Window",
                            JOptionPane.INFORMATION_MESSAGE);
        }
        if(target.equals(aboutItem))
        {
            JOptionPane.showMessageDialog(this, "MultiPlotFrame class\n"
                    + "By: Carmine Lia from PTPLOT\n" + "By: Edward A. Lee "
                    + "and Christopher Hylands\n" + "Version "
                    + PlotBox.PTPLOT_RELEASE + "For more information, see\n"
                    + "http://ptolemy.eecs.berkeley.edu/java/ptplot\n\n"
                    + "Copyright (c) 1997-2005, "
                    + "The Regents of the University of California.",
                    "About Ptolemy Plot", JOptionPane.INFORMATION_MESSAGE);
        }
    }

}
