/*
 * CLIKSimulation.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.simulator;

import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;

import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnElapsedTime;
import javax.robotics.engine.clik.CLIKR;
import javax.robotics.engine.clik.ConstraintVelocity;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.plot.MultiPlot;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.RVector;
/**
 * CLIK simulation helper class for redundant robot manipulators.
 *     
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.2
 * @version 01/nov/06
 *
 */
public class CLIKRSimulation extends Behavior
{
    final Robot3D robot;
    final CLIKR clik;
    final ConstraintVelocity constraint;
    final Trajectory trj;
    WakeupCriterion wakeUp;

    // simulation data
    final int points;
    double time = 0.0;
    final double timeStep;
    Timer sceneTimer;
    double timeDuration;

    // output data
    int i = 0;
    double jointData[][];

    int dof;
    RVector q;
    RVector qp;

    int spaceDim;
    RVector s;
    RVector ds;
    RVector dds;

    MultiPlotPanel plotPanel;
    MultiPlot plotJVel;
    MultiPlot plotEndEffErrPos;
    MultiPlot plotJPos;

    RobotPanel robotPanel;

    static final long updateSchedulerTime = 150l;
    static final int deltaTimer = 20;

    /**
     * @param robot
     * @param clik
     * @param constraint
     * @param trj
     * @param points
     */
    CLIKRSimulation(Robot3D robot, CLIKR clik, ConstraintVelocity constraint,
            Trajectory trj, int points)
    {

        this.robot = robot;
        this.clik = clik;
        this.constraint = constraint;
        this.trj = trj;
        this.points = points;
        this.timeStep = clik.getSampleTime();
        init();
    }

    /**
     * @param robot
     * @param clik
     * @param trj
     * @param points
     */
    public CLIKRSimulation(Robot3D robot, CLIKR clik, Trajectory trj, int points)
    {

        this.robot = robot;
        this.clik = clik;
        this.constraint = null;
        this.trj = trj;
        this.points = points;
        this.timeStep = clik.getSampleTime();
        init();
    }

    @Override
    public void initialize()
    {
        robot.setQ3D(q);
        // end effector path
        trj.interpolate(time, s, ds, dds);

        // clik
        clik.step(s, ds);

        q.get(jointData[i]);

        wakeupOn(wakeUp);

        sceneTimer.scheduleAtFixedRate(new Update3DScene(), 0l,
                updateSchedulerTime);

        createPlotPanel();
        updatePlotPanel();

    }

    @Override
    public void processStimulus(Enumeration arg0)
    {
        if(i >= points - 1)
        {
            this.setEnable(false);
            sceneTimer.cancel();
            plotPanel.fillPlots();
            return;
        }

        time += timeStep;
        wakeupOn(wakeUp);

        // end effector path
        trj.interpolate(time, s, ds, dds);

        clik.step(s, ds);

        i++;
        q.get(jointData[i]);

        updatePlotPanel();
    }
    
    /**
     * @return the panel of plots.
     */
    public final MultiPlotPanel getPlotPanel()
    {
        return plotPanel;
    }

    /**
     * @return the 3D panel of robot. 
     */
    public final RobotPanel getRobotPanel()
    {
        return robotPanel;
    }

    /**
     * @return the computed joints data.
     */
    public final double[][] getJointData()
    {
        return jointData;
    }

    /**
     * Plays the simulation.
     */
    public void play()
    {

    	  Timer playTimer = new Timer();
      PlaySimulation playSimulation = new PlaySimulation(robot, jointData, deltaTimer);
      playTimer.scheduleAtFixedRate(playSimulation, 0l, deltaTimer);
      
    }
    
    /**
     * Starts the simulation.
     */
    public void start()
    {

       setEnable(true);
    }
    
    /**
     * Stops the simulation.
     */
    public void stop()
    {

       setEnable(false);
    }

    
    /**
     * @author Carmine Lia
     *
     */
    final class Update3DScene extends TimerTask
    {

        @Override
        public void run()
        {
            robot.setQ3DOnly(jointData[i]);

        }

    }

    /**
     * @author Carmine Lia
     *
     */
    /*class PlaySimulation extends TimerTask
    {
        int index = 0;

        *//**
         * Main processing method for the SimulationStep object
         *//*
        public void run()
        {
            index += deltaTimer;

            if(index > jointData.length - 1)
            {
                index = jointData.length - 1;
                robot.setQ3DOnly(jointData[index]);
                this.cancel();
            }
            else
                robot.setQ3DOnly(jointData[index]);

        }
    }*/

    /**
     * 
     */
    final void createPlotPanel()
    {

        plotPanel.add(plotJPos);
        plotPanel.add(plotJVel);
        plotPanel.add(plotEndEffErrPos);
        return;
    }

    /**
     * 
     */
    final void updatePlotPanel()
    {
        // plotting joints pos
        plotJPos.addVector(time, q, true);
        // plotting joints vel
        plotJVel.addVector(time, qp, true);
        // plot end effector err pos/orien
        plotEndEffErrPos.addPoint(0, time, clik.getPositionError().length(), true);
    }

    /**
     * 
     */
    private void init()
    {

        timeDuration = points * timeStep;
        dof = robot.getDof();
        jointData = new double[points][dof];

        q = clik.getJointsPosition();
        qp = clik.getJointsVelocity();

        spaceDim = clik.getCartesianSpaceDim();
        s = new RVector(spaceDim);
        ds = new RVector(spaceDim);
        dds = new RVector(spaceDim);

        sceneTimer = new Timer();
        
        wakeUp = new WakeupOnElapsedTime((int) Math
                .round(clik.getSampleTime() * 1000.0));

        // plots panel
        plotPanel = new MultiPlotPanel();
        plotJPos = new MultiPlot("Joints Position", "[s]", "[rad] or [m]", "1",
                "2", "3", "4", "5", "6");
        plotJPos.setXRange(0.0, timeDuration);

        plotJVel = new MultiPlot("Joints Velocity", "[s]", "[rad/s]", "1", "2",
                "3", "4", "5", "6");
        plotJVel.setXRange(0.0, timeDuration);

        plotEndEffErrPos = new MultiPlot("End eff position error", "[s]", "[m]");
        plotEndEffErrPos.setXRange(0.0, timeDuration);

        // robot scene3d panel
        robotPanel = new RobotPanel();
        robotPanel.addRobotToDefaultConfiguration(robot, this);
    }

}
