/*
 * EndEffPosControlSimulation.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.simulator;

import java.util.Enumeration;
import java.util.Timer;
import java.util.TimerTask;

import javax.media.j3d.Behavior;
import javax.media.j3d.WakeupCriterion;
import javax.media.j3d.WakeupOnElapsedTime;
import javax.robotics.engine.controllers.JointController;
import javax.robotics.engine.robots.Robot3D;
import javax.robotics.engine.trajectory.Trajectory;
import javax.robotics.ode.DerivableFunction2;
import javax.robotics.ode.Solver2;
import javax.robotics.plot.MultiPlot;
import javax.robotics.plot.MultiPlotPanel;
import javax.robotics.swing.RobotPanel;
import javax.robotics.vecmath.RVector;
import javax.robotics.vecmath.RVector3d;

/**
 * @author carmine lia
 * @version 01/nov/06
 *
 */
public class JointControlSimulation extends Behavior implements
		DerivableFunction2
{
	Robot3D robot;

	private Solver2 solver;

	private JointController controller;

	private Trajectory trj;

	private WakeupCriterion wakeUp;

	// simulation data
	private int points;

	private double time = 0.0;

	private double timeStep;

	private final Timer sceneTimer;

	// output data
	int i = 0;

	double jointData[][];

	// robot data
	private int dof;

	private final RVector q, qp, tau;

	// joints desired trajectory
	private final RVector qd, qdp, qdpp;

	// plot data
	private final MultiPlotPanel plotPanel;

	private final MultiPlot plotJPos;

	private final MultiPlot plotJVel;

	private final MultiPlot plotJTorque;

	// robot scene 3d
	private final RobotPanel robotPanel;

	private static final long updateSchedulerTime = 150l;

	static final int deltaTimer = 20;

	private double timeDuration;

	// external forces/torques applied
	private final RVector3d Next;

	private final RVector3d Fext;

	public JointControlSimulation(Robot3D robot, JointController controller,
			Solver2 solver, Trajectory trj, int points)
	{
		this.robot = robot;
		this.controller = controller;
		this.solver = solver;
		this.trj = trj;
		this.points = points;
		this.timeStep = solver.getSampleTime();

		this.Fext = new RVector3d();
		this.Next = new RVector3d();

		solver.setTargetFunction(this);

		timeDuration = points * timeStep;
		dof = robot.getDof();
		jointData = new double[points][dof];

		q = new RVector(dof);
		qp = new RVector(dof);
		tau = new RVector(dof);

		qd = new RVector(dof);
		qdp = new RVector(dof);
		qdpp = new RVector(dof);

		sceneTimer = new Timer();
		wakeUp = new WakeupOnElapsedTime((int) Math
				.round(this.timeStep * 1000.0));

		// plots panel
		plotPanel = new MultiPlotPanel();
		plotJPos = new MultiPlot("Joints Position", "[s]", "[rad] or [m]", "1",
				"2", "3", "4", "5", "6");
		plotJPos.setXRange(0.0, timeDuration);

		plotJVel = new MultiPlot("Joints Velocity", "[s]", "[rad/s]", "1", "2",
				"3", "4", "5", "6");
		plotJVel.setXRange(0.0, timeDuration);

		plotJTorque = new MultiPlot("Joints Torque", "[s]", "[N] or [Nm]", "1",
				"2", "3", "4", "5", "6");
		plotJTorque.setXRange(0.0, timeDuration);

		// robot scene3d panel
		robotPanel = new RobotPanel();
		robotPanel.addRobotToDefaultConfiguration(robot, this);
	}

	@Override
	public void initialize()
	{

		time = solver.getTime();
		q.set(solver.getLastValuePosition());
		qp.set(solver.getLastValueVelocity());
		q.get(jointData[i]);
		robot.setQ3D(q);

		// joints desired path
		trj.interpolate(time, qd, qdp, qdpp);

		// 
		tau.set(controller.torqueCmd(qd, qdp, qdpp));
		solver.step(q, qp);

		wakeupOn(wakeUp);

		sceneTimer.scheduleAtFixedRate(new Update3DScene(), 0l,
				updateSchedulerTime);

		createPlotPanel();
		updatePlotPanel();

	}

	@Override
	public void processStimulus(Enumeration arg0)
	{
		if (i >= points - 1)
		{
			this.setEnable(false);
			sceneTimer.cancel();
			plotPanel.fillPlots();
			return;
		}

		time = solver.getTime();
		q.set(solver.getLastValuePosition());
		qp.set(solver.getLastValueVelocity());

		i++;
		q.get(jointData[i]);

		wakeupOn(wakeUp);

		// joints desired path
		trj.interpolate(time, qd, qdp, qdpp);

		// 
		tau.set(controller.torqueCmd(qd, qdp, qdpp));

		solver.step(q, qp);

		updatePlotPanel();
	}

	public final MultiPlotPanel getPlotPanel()
	{
		return plotPanel;
	}

	public final RobotPanel getRobotPanel()
	{
		return robotPanel;
	}

	public final double[][] getJointData()
	{
		return jointData;
	}

	/**
	 * Plays the simulation.
	 */
	public void play()
	{

		Timer playTimer = new Timer();
		PlaySimulation playSimulation = new PlaySimulation(robot, jointData,
				deltaTimer);
		playTimer.scheduleAtFixedRate(playSimulation, 0l, deltaTimer);
	}

	/**
	 * Starts the simulation.
	 */
	public void start()
	{

		setEnable(true);
	}

	/**
	 * Stops the simulation.
	 */
	public void stop()
	{

		setEnable(false);
	}

	final class Update3DScene extends TimerTask
	{

		@Override
		public void run()
		{
			robot.setQ3DOnly(jointData[i]);

		}

	}

	private final void createPlotPanel()
	{

		plotPanel.add(plotJPos);
		plotPanel.add(plotJVel);
		plotPanel.add(plotJTorque);
		return;
	}

	private final void updatePlotPanel()
	{
		// plotting joints pos
		plotJPos.addVector(time, q, true);
		// plotting joints vel
		plotJVel.addVector(time, qp, true);
		// plotting joints torque
		plotJTorque.addVector(time, tau, true);
	}

	// implementation of derivable function of Solver
	public RVector xDot2(RVector q, RVector qp)
	{
		return robot.acceleration(q, qp, tau, Fext, Next);
	}
}
