/*
 * JointSpinnerPanel.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.robotics.engine.robots.Robot3D;
import javax.robotics.vecmath.RVector;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import javolution.util.FastList;

/**
 * This class creates a panel of spinners to move the joints of Robot.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 * @version 15/10/2005
 */
public class JointSpinnerPanel extends JPanel implements ChangeListener
{
    /**
     * 
     */
    private static final long serialVersionUID = 3906084555432540466L;
    private int dof = 0;
    private double step = 0.01;
    private Robot3D robot;
    private RVector q0, qMin, qMax;;
    private GridBagLayout gridbag;
    private GridBagConstraints c;
    private FastList<JSpinner> array;
    private JButton set, reset;
    private JLabel errorLabel; 

    /**
     * Creates the panel of spinners.
     * 
     * @param robot
     *            the Robot to link to.
     * @param stepSize the minumum size of spinners step.
     */
    public JointSpinnerPanel(Robot3D robot, double stepSize)
    {
        super();

        this.robot = robot;
        step = stepSize;
        dof = robot.getDof();
        q0 = robot.getQ();
        qMin = robot.getQMin();
        qMax = robot.getQMax();
        
        errorLabel = new JLabel("");
        errorLabel.setForeground(Color.RED);
        
        array = new FastList<JSpinner>(dof);

        gridbag = new GridBagLayout();
        c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.gridx = 0;
        setLayout(gridbag);
        // setBackground(Color.black);
        setBorder(BorderFactory.createCompoundBorder(BorderFactory
                .createTitledBorder(BorderFactory.createLineBorder(Color.gray),
                        "Joints Position"), BorderFactory.createEmptyBorder(2,
                5, 2, 5)));
        final Insets ins = new Insets(1, 1, 1, 1);
        for (int i = 0; i < dof; i++)
        {
            JSpinner s = createJSpinner("joint" + (i + 1), qMin.getElement(i), qMax.getElement(i), q0.getElement(i), step);

            c.gridy = i;
            c.insets = ins;
            gridbag.setConstraints(s, c);
            s.setName(Integer.toString(i + 1));
            s.addChangeListener(this);
            add(s);
            array.add(s);
        }
        c.gridy = dof;
        add(createButtonPanel(), c);
        c.gridy = dof+1;
        add(errorLabel, c);
     }

  
    /**
     * Sets the value of spinners.
     * 
     * @param q
     *            the vector of new values.
     */
    public final void setValue(RVector q)
    {
        
        for (int i = 0; i < dof; i++)
        {
            final JSpinner s = array.get(i);
            if(q.getElement(i) < qMin.getElement(i) || q.getElement(i) > qMax.getElement(i))
            {
                errorLabel.setText("joint "+ (i+1) +" not in range");
                return;
            }
            s.setValue((Double) q.getElement(i));
            
        }
        errorLabel.setText("");
    }

    /**
     * Gets the value of spinners.
     * 
     * @return the joints position.
     */
    public RVector getValue()
    {
        final RVector q = new RVector(dof);
        for (int i = 0; i < dof; i++)
        {
            final JSpinner s = array.get(i);
            q.setElement(i, (Double) s.getValue());
        }
        return q;
    }
    /**
     * Updates the values of spinners to current robot position.
     */
    public void updateValue()
    {
        setValue(robot.getQ());
    }

   
    /**
     * Sets the values of spinners to zero.
     */
    public void resetValue()
    {
        setValue(q0);
    }

    public void stateChanged(ChangeEvent e)
    {
        final JSpinner source = (JSpinner) e.getSource();
        final Integer joint = new Integer(source.getName());
        robot.setQ3D((Double) source.getValue(), joint);
        errorLabel.setText("");
   }

    private JSpinner createJSpinner(String tip, double min, double max, double value, double step)
    {
        final JSpinner spinner = new JSpinner(new SpinnerNumberModel(value, min,  max, step));
        
        spinner.addChangeListener(this);
        spinner.setToolTipText(tip);
        // System.out.println(spinner.getPreferredSize());
        spinner.setPreferredSize(new Dimension(100, 20));
        return spinner;
    }

    private JPanel createButtonPanel()
    {
        final JPanel bp = new JPanel();
        set = new JButton("Set");
        set.setToolTipText("Update the values to the current joints position");
        set.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                updateValue();
            }
        });
        bp.add(set);
        reset = new JButton("Reset");
        reset.setToolTipText("Sets the values to initial position");
        reset.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
               resetValue();
            }
        });
        bp.add(reset);
        return bp;
    }


    /**
     * Gets the minimum step of spinners.
     * @return Returns the step value.
     */
    public final double getStepSize()
    {
        return step;
    }

}
