/*
 * Matrix.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.vecmath;

import java.io.PrintStream;
import java.util.Locale;

import javax.robotics.vecmath.jama.EigenvalueDecomposition;
import javax.robotics.vecmath.jama.LUDecomposition;
import javax.robotics.vecmath.jama.QRDecomposition;
import javax.robotics.vecmath.jama.SingularValueDecomposition;
import javax.vecmath.GMatrix;
import javax.vecmath.GVector;
import javax.vecmath.MismatchedSizeException;

/**
 * A double precision, general, dynamically-resizable, two-dimensional matrix
 * class. Row and column numbering begins with zero. The representation is row
 * major. This class extends printing functionality of GMatrix class
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.0.1
 * @version 5/10/2005
 */
public class Matrix extends GMatrix
{

    /**
     * 
     */
    private static final long serialVersionUID = 3906364909473575728L;

    private static final double eps = 2E-15;
    
    private static final String strLineSeparator = "line.separator";
    private static final String newLine = System.getProperty(strLineSeparator);
    private static final String formatString = "%9.4f ";
    
    /**
     * Type of matrix.
     * 
     * @see #eig(Matrix, Matrix, RVector, RVector)
     */
    public static final short SEMIDEFINITE_NEGATIVE = 0;
    /**
     * Type of matrix.
     * 
     * @see #eig(Matrix, Matrix, RVector, RVector)
     */
    public static final short DEFINITE_NEGATIVE = 1;
    /**
     * Type of matrix.
     * 
     * @see #eig(Matrix, Matrix, RVector, RVector)
     */
    public static final short SEMIDEFINITE_POSITIVE = 2;
    /**
     * Type of matrix.
     * 
     * @see #eig(Matrix, Matrix, RVector, RVector)
     */
    public static final short DEFINITE_POSITIVE = 3;

    /**
     * Constructs a new matrix with the same values as the GMatrix parameter.
     * 
     * @param matrix
     *            the source matrix
     */
    public Matrix(GMatrix matrix)
    {
        super(matrix);
    }

    /**
     * Constructs a new matrix with the same values as the Matrix parameter.
     * 
     * @param matrix
     *            the source matrix
     */
    public Matrix(Matrix matrix)
    {
        super(matrix);
    }

    /**
     * Constructs an n by n identity matrix.
     * 
     * @param n
     *            number of rows and columns in this matrix.
     */
    public Matrix(int n)
    {
        super(n, n);
    }
    /**
     * Constructs an nRow by NCol identity matrix. Note that because row and
     * column numbering begins with zero, nRow and nCol will be one larger than
     * the maximum possible matrix index values.
     * 
     * @param nRow
     *            number of rows in this matrix.
     * @param nCol
     *            number of columns in this matrix.
     */
    public Matrix(int nRow, int nCol)
    {
        super(nRow, nCol);
    }
    /**
     * Constructs an n by n square matrix initialized to the values in the
     * matrix array. The array values are copied in one row at a time in row
     * major fashion. The array should be at least n*n in length. Note that
     * because row and column numbering begins with zero, nRow and nCol will be
     * one larger than the maximum possible matrix index values.
     * 
     * @param n
     *            number of rows and columns in this matrix.
     * @param matrix
     *            the array that specifies a matrix in row major fashion
     */
    public Matrix(int n, double[] matrix)
    {
        super(n, n, matrix);
    }
    /**
     * Constructs an nRow by nCol matrix initialized to the values in the matrix
     * array. The array values are copied in one row at a time in row major
     * fashion. The array should be at least nRow*nCol in length. Note that
     * because row and column numbering begins with zero, nRow and nCol will be
     * one larger than the maximum possible matrix index values.
     * 
     * @param nRow
     *            number of rows in this matrix.
     * @param nCol
     *            number of columns in this matrix.
     * @param matrix
     *            the array that specifies a matrix in row major fashion
     */
    public Matrix(int nRow, int nCol, double[] matrix)
    {
        super(nRow, nCol, matrix);
    }

    /**
     * Prints to standard out
     */
    public void printf()
    {
        this.printf(System.out, formatString);
    }

    /**
     * Prints to standard out according to the format string.
     * <p>
     * The format specifiers for numeric types have the following syntax:
     * 
     * <code>
     *    %[width][.precision]conversion 
     * </code>
     * 
     * <ul>
     * <li> The optional <b><code>width</code></b> is a non-negative decimal
     * integer indicating the minimum number of characters to be written to the
     * output. </li>
     * <li> The optional <b><code>precision</code></b> is a non-negative
     * decimal integer usually used to restrict the number of characters. The
     * specific behavior depends on the conversion. </li>
     * <li> The required <b><code>conversion</code></b> is a character
     * indicating how the argument should be formatted. The set of valid
     * conversions for a given argument depends on the argument's data type.
     * </li>
     * </ul>
     * <p>
     * Example: </br><code>v.printf("%5.3f");</code>
     * </p>
     * 
     * @param format
     *            The format string
     * @see PrintStream#format(java.lang.String, java.lang.Object[])
     */
    public void printf(String format)
    {
        this.printf(System.out, format);
    }

    /**
     * Prints to <code>PrintStream</code>
     * 
     * @param out
     *            The <code>PrintStream</code> object.
     */
    public void printf(PrintStream out)
    {
        this.printf(out, formatString);
    }

    /**
     * Prints to <code>PrintStream</code> according to the format string
     * 
     * @param out
     *            The <code>PrintStream</code> object.
     * @param format
     *            The format string.
     * @see #printf(String)
     */
    public void printf(PrintStream out, String format)
    {
        final int nRow = this.getNumRow();
        final int nCol = this.getNumCol();
        for (int i = 0; i < nRow; i++)
        {
            for (int j = 0; j < nCol; j++)
                out.printf(Locale.US, format, this.getElement(i, j));
            out.println(newLine);
        }
    }

    /**
     * Returns a string that contains the values of this Matrix.
     * 
     * @return The string.
     */
    public String toString()
    {
        return this.toString(formatString);
    }

    /**
     * Returns a string that contains the values of this Matrix, according to
     * the format string
     * 
     * @param format
     *            The format string
     * @return The string
     * @see #printf(String)
     */
    public String toString(String format)
    {
        final int nRow = this.getNumRow();
        final int nCol = this.getNumCol();

        final StringBuffer buffer = new StringBuffer(nRow * nCol * 8);

        int i, j;

        for (i = 0; i < nRow; i++)
        {
            for (j = 0; j < nCol; j++)
            {
                buffer.append(String.format(Locale.US, format, this.getElement(i, j)));
                // .append(" ");
            }
            buffer.append(newLine);
        }

        return buffer.toString();
    }
    // added function
    /*
     * ---------------- Special function ------------------
     */
    /**
     * Finds the singular value of this matrix. <b>This is the jacobi version
     * </b>
     * 
     * <p>
     * Author: G. Jungman, Java version: Carmine Lia
     * </p>
     * <p>
     * Algorithm due to J.C. Nash, Compact Numerical Methods for Computers (New
     * York: Wiley and Sons, 1979), chapter 3. See also Algorithm 4.1 in James
     * Demmel, Kresimir Veselic, "Jacobi's Method is more accurate than QR",
     * Lapack Working Note 15 (LAWN15), October 1989. Available from netlib.
     * http://netlib.org/lapack/ in the lawns or lawnspdf directories
     * </p>
     * <p>
     * Based on code by Arthur Kosowsky, Rutgers University
     * kosowsky@physics.rutgers.edu
     * </p>
     * <p>
     * Another relevant paper is, P.P.M. De Rijk, "A One-Sided Jacobi Algorithm
     * for computing the singular value decomposition on a vector computer",
     * SIAM Journal of Scientific and Statistical Computing, Vol 10, No 2, pp
     * 359-371, March 1989.
     * </p>
     * 
     * @param s
     *            RVector of n singular values
     * @return Rank of this matrix, if negative an error occurred.
     * 
     */
    public int svd(RVector s)
    {

        final int m = this.getNumRow();
        final int n = this.getNumCol();

        final  Matrix A, Q;

        A = new Matrix(this);
        
        if(m < n)
        {
            A.transpose();
        }

       
        Q = new Matrix(n, n);
        Q.setIdentity();

        // inizializzazione del contatore (count) delle rotazioni e del
        // contatore delle passate (sweep)
        int count = n * (n - 1) / 2;
        final int iCount = count;
        int sweep = 0;

        // tolleranza per esaurire il ciclo
        final  double tolerance = 1e-10;

        // il numero minimo delle passate ? 12
        final  int sweepmax = Math.max(n, 12);
        
        final  GVector cj = new GVector(m);
        final  GVector ck = new GVector(m);
       
        
        
        // ciclo per l'ortogonalizzazione della matrice A
        while (count > 0 && sweep <= sweepmax)
        {
            count = iCount;

            for (int j = 0; j < n - 1; j++)
            {
                A.getColumn(j, cj);
                final  double a = cj.norm();
                double q = a * a;
                
                for (int k = j + 1; k < n; k++)
                {
                    double cos = 0.0;
                    double sin = 0.0;
                    
                    A.getColumn(k, ck);

                    final  double p = cj.dot(ck);
                    final  double b = ck.norm();
                    final  double r = b * b;

                    if(b <= tolerance * a)
                    {
                        count--;
                        continue;
                    }

                    if(Math.abs(p) <= tolerance * a * b)
                    {
                        count--;
                        continue;
                    }

                    // calcolo degli angoli di rotazione
                    if(q < r)
                    {
                        cos = 0.0;
                        sin = 1.0;
                    }
                    else
                    {
                        q -= r;
                        final double v = Math.sqrt(4.0 * p * p + q * q);
                        cos = Math.sqrt((v + q) / (2.0 * v));
                        sin = p / (v * cos);
                    }

                    // rotazione di A
                    for (int i = 0; i < m; i++)
                    {

                        final  double Aik = A.getElement(i, k);
                        final  double Aij = A.getElement(i, j);
                        A.setElement(i, j, Aij * cos + Aik * sin);
                        A.setElement(i, k, -Aij * sin + Aik * cos);

                    }

                    // rotazione di Q
                    for (int i = 0; i < n; i++)
                    {

                        final  double Qik = Q.getElement(i, k);
                        final  double Qij = Q.getElement(i, j);
                        Q.setElement(i, j, Qij * cos + Qik * sin);
                        Q.setElement(i, k, -Qij * sin + Qik * cos);

                    }

                }
            }
            sweep++;
        }
        // fine dell'ortogonalizzazione delle matrici

        // Calcolo dei valori singolari
        double prev_norm = -1.0;
        int rank = 0;
        final GVector col = new GVector(m);
        
        for (int j = 0; j < n; j++)
        {
            A.getColumn(j, col);
            final double norm = col.norm();

            if((norm == 0.0) || (prev_norm == 0.0)
                    || (j > 0 && norm <= tolerance * prev_norm))
            {
                s.setElement(j, 0.0);
                // col.zero();
                prev_norm = 0.0;
            }
            else
            {
                rank++;
                s.setElement(j, norm);
                // col.normalize();
                prev_norm = norm;
            }
        }
        if(count > 0)
        {
            return -1;
        }

        return rank;
    }

    /**
     * Finds Singular Values Decomposition SVD of this matrix.
     * <p>
     * Factorise this M x N matrix, </br> <code>this = U Q V^T</code> </br>
     * where U is a column-orthogonal M x N matrix (U^T U = I), Q is a diagonal
     * N x N matrix, and V is an N x N orthogonal matrix (V^T V = V V^T = I).
     * </p>
     * 
     * @param Q
     *            the n x n diagonal matrix of singular values
     * @param U
     *            the column-orthogonal m x n matrix
     * @param V
     *            the orthogonal n x n matrix
     * @return the rank of this matrix.     
     * @see SingularValueDecomposition
     */
    public int svd(Matrix Q, Matrix U, Matrix V)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n > m)
            throw new MismatchedSizeException("SVD works with NumRow >= NumCol");

        final double A[][] = new double[m][n];
        this.get(A);

        final SingularValueDecomposition SVD = new SingularValueDecomposition(A, m, n);
        Q.set(SVD.getS());
        U.set(SVD.getU());
        V.set(SVD.getV());
        return SVD.rank();
    }
    /**
     * Eigenvalues and eigenvectors of a nxn real matrix.
     * 
     * @param D
     *            the nxn block diagonal eigenvalue matrix
     * @param V
     *            the nxn eigenvector matrix
     * @see EigenvalueDecomposition
     */
    public void eig(Matrix D, Matrix V)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != m)
            throw new MismatchedSizeException("The matrix is not square");

        final double A[][] = new double[m][n];
        this.get(A);

        final EigenvalueDecomposition EIG = new EigenvalueDecomposition(A, n);
        D.set(EIG.getD());
        V.set(EIG.getV());
    }

    /**
     * Eigenvalues and eigenvectors of a nxn real matrix.
     * 
     * @param D
     *            the nxn block diagonal eigenvalue matrix
     * @param V
     *            the nxn eigenvector matrix
     * @param d
     *            the real part of the eigenvalues
     * @param e
     *            the imaginary parts of the eigenvalues
     * @see EigenvalueDecomposition
     */
    public void eig(Matrix D, Matrix V, RVector d, RVector e)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != m)
            throw new MismatchedSizeException("The matrix is not square");

        final double A[][] = new double[m][n];
        this.get(A);

        final EigenvalueDecomposition EIG = new EigenvalueDecomposition(A, n);
        D.set(EIG.getD());
        V.set(EIG.getV());
        d.set(EIG.getRealEigenvalues());
        e.set(EIG.getImagEigenvalues());
    }
    /**
     * Eigenvalues of a nxn real matrix.
     * 
     * @param d
     *            the real part of the eigenvalues
     * @see EigenvalueDecomposition
     */
    public void eig(RVector d)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != m)
            throw new MismatchedSizeException("The matrix is not square");

        final double A[][] = new double[m][n];
        this.get(A);

        final EigenvalueDecomposition EIG = new EigenvalueDecomposition(A, n);
        d.set(EIG.getRealEigenvalues());
       
    }
    /**
     * Eigenvalues of a nxn real matrix.
     * 
     * @return d
     *            the real part of the eigenvalues
     * @see EigenvalueDecomposition
     */
    public RVector eig()
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != m)
            throw new MismatchedSizeException("The matrix is not square");

        final double A[][] = new double[m][n];
        this.get(A);

        final EigenvalueDecomposition EIG = new EigenvalueDecomposition(A, n);
        final RVector d = new RVector(n);
        d.set(EIG.getRealEigenvalues());
        return d;
       
    }

    /**
     * Check the eigenvalues.
     * 
     * @param d
     *            the vector of real part of eigenvalues.
     * @return the type of matrix: (semi)-definite negative/positive otherwise
     *         -1.
     * @see #DEFINITE_NEGATIVE
     * @see #DEFINITE_POSITIVE
     * @see #SEMIDEFINITE_NEGATIVE
     * @see #SEMIDEFINITE_POSITIVE
     */
    public static short checkEigenValue(RVector d)
    {
        final int n = d.getSize();
        // check type
        int pos = 0;
        int neg = 0;
        int zero = 0;
        // ????
        final double tol = d.norm() * eps * n;
        for (int i = 0; i < n; i++)
        {
            if(d.getElement(i) == 0.0 || Math.abs(d.getElement(i)) < tol)
                zero++;
            if(d.getElement(i) > 0.0)
                pos++;
            if(d.getElement(i) < 0.0)
                neg++;

        }
        if(pos > 0 && neg == 0)
            
            if(zero == 0)
                return DEFINITE_POSITIVE;
            else
                return SEMIDEFINITE_POSITIVE;
        if(pos == 0 && neg > 0)
            if(zero == 0)
                return DEFINITE_NEGATIVE;
            else
                return SEMIDEFINITE_NEGATIVE;
        return -1;

    }
    /**
     * Computes pseudo inverse of this matrix using SVD decomposition.
     * 
     * @return the pseudo inverse matrix.
     */
    public Matrix pseudoInverse()
    {
        //this function is like inverse(0.0, 0.0) 
        
        final int m = this.getNumRow();
        final int n = this.getNumCol();

        if(n > m)
        {
            final Matrix M = new Matrix(this);
            M.transpose();
            final Matrix X = M.pseudoInverse();
            X.transpose();
            return X;
        }

        final Matrix Q = new Matrix(n, n);
        final Matrix U = new Matrix(m, n);
        final Matrix V = new Matrix(n, n);
        final Matrix X = new Matrix(n, m);
        final int r = this.svd(Q, U, V);
       // double tol = Math.max(m, n) * Q.getElement(1, 1) * eps;

        // conteggio dei sv != 0 calcolo del rango
     /*   int r = 0;
        for (int i = 0; i < Q.getNumRow(); i++)
            if(Q.getElement(i, i) > tol)
                r++;
      */
        if(r != 0)
        {
            final Matrix Q0 = new Matrix(r, r);
            for (int i = 0; i < r; i++)
                Q0.setElement(i, i, 1.0 / Q.getElement(i, i));

            final Matrix V0 = new Matrix(V.getNumRow(), r);
            final Matrix U0 = new Matrix(U.getNumRow(), r);
            V.getSubMatrix(0, V.getNumRow() - 1, 0, r - 1, V0);
            U.getSubMatrix(0, U.getNumRow() - 1, 0, r - 1, U0);
            X.mulTransposeRight(Q0, U0);
            X.mul(V0, X);

        }
        else
        {
            X.setZero();
        }
        return X;

    }
    /**
     * Computes Damped Least Square inverse of this matrix using SVD decomposition.
     * <p>
     *  A<sup>-1</sup> = A<sup>T</sup>*(A*A<sup>T</sup> + &lambda;<sup>2</sup>*I)<sup>-1</sup>
     *  </br>where &lambda;<sup>2</sup> is the damped factor equals 1 - (&sigma;<sub>r</sub>/&epsilon;)*&lambda;<sup>2</sup><sub>max</sub>
     *  if &sigma;<sub>r</sub>&lt; &epsilon;, equals 0 if &sigma;<sub>r</sub>&ge; &epsilon;; &sigma;<sub>r</sub> is the smallest not null singular value.
     *  </p>
     * @param eps the area of singular region.
     * @param lambdaMax the maximum damped factor.
     * @return the inverse matrix, zero matrix if rank=0.
     */
    public Matrix inverse(double eps, double lambdaMax)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n > m)
        {
           
            final Matrix M = new Matrix(this);
            M.transpose();
            final Matrix X = M.inverse(eps, lambdaMax);
            X.transpose();
            return X;
        }

        final Matrix Q = new Matrix(n, n);
        final Matrix U = new Matrix(m, n);
        final Matrix V = new Matrix(n, n);
        final Matrix X = new Matrix(n, m);
        // rank
        final int r = this.svd(Q, U, V);
        
        if(r != 0)
        {
            final double smallestSingValue = Q.getElement(r-1,r-1);
            double lambda = 0.0;
               
            if(smallestSingValue < eps)
            {
                lambdaMax *= lambdaMax;
                lambda = (1.0-Math.pow(smallestSingValue/eps,2))*lambdaMax;
            }
            
            final Matrix Q0 = new Matrix(r, r);
            for (int i = 0; i < r; i++)
                Q0.setElement(i, i, Q.getElement(i, i) / (Q.getElement(i, i)*Q.getElement(i, i)+lambda));

            final Matrix V0 = new Matrix(V.getNumRow(), r);
            final Matrix U0 = new Matrix(U.getNumRow(), r);
            V.getSubMatrix(0, V.getNumRow() - 1, 0, r - 1, V0);
            U.getSubMatrix(0, U.getNumRow() - 1, 0, r - 1, U0);
            final Matrix XX = new Matrix(r, U0.getNumRow());
            XX.mulTransposeRight(Q0, U0);
            X.mul(V0, XX);

        }
        else
        {
            X.setZero();
        }
        return X;
    }
    
    /**
     * Computes inverse or pseudoinverse of this matrix
     * 
     * @return inverse if this is square, pseudoinverse otherwise.
     */
    public Matrix inverse()
    {
        final int n = this.getNumCol();
        final int m = this.getNumRow();
        final Matrix inverse = new Matrix(n, m);
        final double matrix[][];

        // create input identity matrix
        final double B[][] = new double[m][m];

        for (int i = 0; i < m; i++)
            B[i][i] = 1.0;

     
        if(n == m)
        {
            matrix = new double[m][n];
            this.get(matrix);
            final LUDecomposition LU = new LUDecomposition(matrix, m, n);
            inverse.set(LU.solve(B, m, m));
        }
        else
        {
            matrix = new double[m][n];
            this.get(matrix);
            final QRDecomposition QR = new QRDecomposition(matrix, m, n);
            inverse.set(QR.solve(B, m, m));
        }
        return inverse;
    }

    /**
     * Computes the solution of Lyapunov equation using eigenvalue
     * decomposition. The return matrix is the solution of equation: </br>
     * <code> A*X + X*A' = -C </code> </br> where the matrix A is this matrix,
     * the matrix X is the solution and C is the input matrix. Both the matrices
     * must be square with same dimensions.
     * <p>
     * <b>Warning!</b> the function fail if the eigenvalue of A are complex.
     * </p> 
     * @param C
     *            input matrix of equation
     * @return the matrix solution of equation.
     */
    public Matrix lyap(Matrix C)
    {

        final int na = this.getNumCol();
        final int ma = this.getNumRow();

        final int nc = C.getNumCol();
        final int mc = C.getNumRow();

        if((ma != na) || (mc != nc))
            throw new MismatchedSizeException("The matrices must be square.");
        if((mc != ma) || (nc != na))
            throw new MismatchedSizeException(
                    "The matrices dimension are not agree.");

        final Matrix X = new Matrix(na);
        final Matrix D = new Matrix(na);
        final Matrix Dt = new Matrix(na);
        final Matrix V = new Matrix(na);
        final Matrix Vi = new Matrix(na);
        final Matrix ones = new Matrix(na);
        
        ones.setOnes();
        this.eig(D, V);
        Vi.invert(V);
        D.mul(ones);
        X.mulTransposeRight(C, Vi);
        Vi.mul(X);
        Dt.transpose(D);
        D.add(D, Dt);
        X.set(Vi.arrayRightDivide(D));
        X.mulTransposeRight(X, V);
        V.mul(X);
        X.negate(V);
        return X;

    }
    /*
     * -------------- Useful functions
     */
    /**
     * Sets all elements of this matrix to one
     */
    public void setOnes()
    {
        final int n = this.getNumCol();
        final int m =  this.getNumRow(); 
        for (int i = 0; i < m; i++)
            for (int j = 0; j < n; j++)
                this.setElement(i, j, 1.0);
    }
    /**
     * Gets main diagonal of this matrix.
     * 
     * @return RVector equals diagonal of matrix
     */
    public RVector getDiagonal()
    {
        final int n = this.getNumCol();
        final int m = this.getNumRow();
        final int size = Math.min(m, n);
        final RVector diag = new RVector(size);
        for (int i = 0; i < size; i++)
            diag.setElement(i, this.getElement(i, i));

        return diag;
    }

    /**
     * Sets the diagonal of this matrix. The number of elements placed is the
     * minimum between this.nRow, this.nCol, RVector.size
     * 
     * @param diag
     *            elements to be placed
     */
    public void setDiagonal(RVector diag)
    {
        final int n = this.getNumCol();
        final int m = this.getNumRow();

        final int size = Math.min(diag.getSize(), Math.min(m, n));

        for (int i = 0; i < size; i++)
            this.setElement(i, i, diag.getElement(i));

    }

    /**
     * Gets this matrix as mono dimensional array
     * 
     * @param matrix
     *            the array
     */
    public void get(double[] matrix)
    {
        int k = 0;
        final int n = this.getNumCol();
        final int m = this.getNumRow();

        for (int i = 0; i < m; i++)
            for (int j = 0; j < n; j++)
            {
                matrix[k] = this.getElement(i, j);
                k++;
            }

    }

    /**
     * Gets this matrix as bi-dimensional array
     * 
     * @param matrix
     *            the array copied
     */
    public void get(double[][] matrix)
    {
        final int n = this.getNumCol();
        final int m = this.getNumRow();

        for (int i = 0; i < m; i++)
            for (int j = 0; j < n; j++)
                matrix[i][j] = this.getElement(i, j);

    }
    /**
     * Sets this matrix from bi-dimensional array
     * 
     * @param matrix
     *            the source array
     */
    public void set(double[][] matrix)
    {
        final int n = this.getNumCol();
        final int m = this.getNumRow();

        for (int i = 0; i < m; i++)
            for (int j = 0; j < n; j++)
                this.setElement(i, j, matrix[i][j]);

    }
    /**
     * Gets sub matrix from this Matrix.
     * 
     * @param startRow
     * @param endRow
     * @param startCol
     * @param endCol
     * @param M
     *            the sub matrix
     */
    public void getSubMatrix(int startRow, int endRow, int startCol,
            int endCol, Matrix M)
    {
        int k = 0;
        int l = 0;
        for (int i = startRow; i <= endRow; i++)
        {
            for (int j = startCol; j <= endCol; j++)
            {
                M.setElement(k, l, this.getElement(i, j));
                l++;
            }
            k++;
            l = 0;
        }
    }
    /**
     * Element-by-element right division, C = this./B
     * 
     * @param B
     *            another matrix
     * @return this./B
     */

    public Matrix arrayRightDivide(Matrix B)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        
        if(n != B.getNumCol() || m != B.getNumRow())
            throw new MismatchedSizeException();

        final Matrix C = new Matrix(m, n);
        for (int i = 0; i < m; i++)
        {
            for (int j = 0; j < n; j++)
            {
                C.setElement(i, j, this.getElement(i, j) / B.getElement(i, j));
            }
        }
        return C;
    }

    /**
     * Element-by-element right division in place, this = this./B
     * 
     * @param B
     *            another matrix
     * @return this./B
     */

    public Matrix arrayRightDivideEquals(Matrix B)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        
        if(n != B.getNumCol() || m != B.getNumRow())
            throw new MismatchedSizeException();

        for (int i = 0; i < m; i++)
        {
            for (int j = 0; j < n; j++)
            {
                this.setElement(i, j, this.getElement(i, j)
                        / B.getElement(i, j));
            }
        }
        return this;
    }

    /**
     * Element-by-element left division, C = this.\B
     * 
     * @param B
     *            another matrix
     * @return this.\B
     */

    public Matrix arrayLeftDivide(Matrix B)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != B.getNumCol() || m != B.getNumRow())
            throw new MismatchedSizeException();

        final Matrix C = new Matrix(m, n);
        for (int i = 0; i < m; i++)
        {
            for (int j = 0; j < n; j++)
            {
                C.setElement(i, j, B.getElement(i, j) / this.getElement(i, j));
            }
        }
        return C;
    }

    /**
     * Element-by-element left division in place, this = this.\B
     * 
     * @param B
     *            another matrix
     * @return this.\B
     */
    public Matrix arrayLeftDivideEquals(Matrix B)
    {
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n != B.getNumCol() || m != B.getNumRow())
            throw new MismatchedSizeException();

        for (int i = 0; i < m; i++)
        {
            for (int j = 0; j < n; j++)
            {
                this.setElement(i, j, B.getElement(i, j)
                        / this.getElement(i, j));
            }
        }
        return this;
    }
    /**
     * Check if this matrix is symmetric
     * 
     * @return true if this matrix is symmetric
     */
    public boolean isSymmetric()
    {
        
        boolean issym = true;
        final int m = this.getNumRow();
        final int n = this.getNumCol();
        if(n == m)
        {
            final double tol = eps*Math.max(n,m)*100.0;

            for (int i = 0; i < m; i++)
                for (int j = i + 1; j < n; j++)
                    if(i != j)
                    {
                        final double diff = (this.getElement(i, j) - this.getElement(j, i));
//                      ???
                        
                        if(Math.abs(diff) > tol)
                        {
                            issym = false;
                            break;
                        }
                    }

        }
        else
            issym = false;
        
        return issym;

    }
    
    /**
     * Sets the value of this matrix to the result of multiplying itself with diagonal matrix D (this = this * D). 
     * @param D the input diagonal matrix.
     */
    public void mul(DMatrix D)
    {
        final int size = D.getSize();
        final int NCol = this.getNumCol();
        if( size != NCol)
            throw new MismatchedSizeException("The matrices dimension are not agree.");
        
       for(int i=0; i < size; i++)
           for(int j=0; j < NCol; j++)
               this.setElement(i, j, D.getElement(j)*this.getElement(i,j)); 
    }
    
    /**
     * Splits columns of this matrix in multiple vectors.
     * @param v the destination array of vectors
     */
    public void splitColumns(RVector... v)
    {
        final int numCol = v.length;
        final int numRow = this.getNumRow();
        if(numCol > this.getNumCol())
            throw new MismatchedSizeException("The number of vectors must be <= the numbers of matrix columns.");
            
        for(int j = 0; j<numCol; j++)
            for(int i=0; i<numRow; i++)
                v[j].setElement(i,this.getElement(i,j));
    }
    /**
     * Splits columns of this matrix in multiple arrays.
     * @param v the destination array of arrays
     */
    public void splitColumns(double[]... v)
    {
        final int numCol = v.length;
        final int numRow = this.getNumRow();
        if(numCol > this.getNumCol())
            throw new MismatchedSizeException("The number of vectors must be <= the numbers of matrix columns.");
            
        for(int j = 0; j<numCol; j++)
            for(int i=0; i<numRow; i++)
                v[j][i] =this.getElement(i,j);
    }
    /**
     * Splits rows of this matrix in multiple vectors.
     * @param v the destination array of vectors
     */
    public void splitRows(RVector... v)
    {
        final int numRow = v.length;
        final int numCol = this.getNumCol();
        if(numRow > this.getNumRow())
            throw new MismatchedSizeException("The number of vectors must be <= the numbers of matrix rows.");
            
        for(int i = 0; i<numRow; i++)
            for(int j=0; j<numCol; j++)
                v[i].setElement(j,this.getElement(i,j));
    }
    
    /**
     * Splits rows of this matrix in multiple arrays.
     * @param v the destination array of arrays
     */
    public void splitRows(double[]... v)
    {
        final int numRow = v.length;
        final int numCol = this.getNumCol();
        if(numRow > this.getNumRow())
            throw new MismatchedSizeException("The number of vectors must be <= the numbers of matrix rows.");
            
        for(int i = 0; i<numRow; i++)
            for(int j=0; j<numCol; j++)
                v[i][j] =this.getElement(i,j);
    }
}
