/*
 * XmlSpline.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.robotics.vecmath.Matrix;
import javax.robotics.vecmath.Quaternion;
import javax.robotics.vecmath.RMatrix3d;

import org.apache.xmlbeans.XmlException;

import pathML.MatrixType;
import pathML.PathConfigurationType;
import pathML.SplineType;
import pathML.QuaternionType;
import pathML.PathConfigurationDocument;
import pathML.PathConfigurationType.Type.Enum;
/**
 * XML handler class for spline path configuration file.
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 * @version 30/10/2005
 */
public class XmlSpline
{

    private PathConfigurationDocument splineDoc = null;
    private File splineXml;
    private InputStream splineXmlStream;
    private String description, name;
    private Enum type;
    private Matrix position, orientation;
    private int nPath, spaceDim;
    private boolean noOrientation;

    /**
     * Constructor of the xml handler.
     * 
     * @param splineXml
     *            the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlSpline(String splineXml) throws XmlException, IOException
    {
        this(new File(splineXml));
        
    }
    /**
     * Constructor of the xml handler.
     * 
     * @param splineXml
     *            the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlSpline(File splineXml) throws XmlException, IOException
    {
        this.splineXml = splineXml;
        splineDoc = PathConfigurationDocument.Factory.parse(this.splineXml);
        if(!splineDoc.validate())
            throw new XmlException("The file " + splineXml.getName()
                    + " isn't a valid xml");

        getGeneralData();
        if(!getPathData())
            throw new XmlException("Invalid path data");
        
        // System.out.println(position);
        // System.out.println(orientation);
    }
    
    /**
     * Constructor of the xml handler.
     * 
     * @param splineXml
     *            the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlSpline(InputStream splineXml) throws XmlException, IOException
    {
        this.splineXmlStream = splineXml;
        splineDoc = PathConfigurationDocument.Factory.parse(this.splineXmlStream);
        if(!splineDoc.validate())
            throw new XmlException("The input stream isn't a valid xml");

        getGeneralData();
        if(!getPathData())
            throw new XmlException("Invalid path data");
        
        // System.out.println(position);
        // System.out.println(orientation);
    }
    
    /**
     * read the information about path
     * 
     * @throws XmlException
     */
    private void getGeneralData() throws XmlException
    {
        description = splineDoc.getPathConfiguration().getDescription();
        name = splineDoc.getPathConfiguration().getName();
        type = splineDoc.getPathConfiguration().getType();
        noOrientation = !splineDoc.getPathConfiguration().getOrientation();
        spaceDim = splineDoc.getPathConfiguration().getSpaceDimension();

        if(!pathML.PathConfigurationType.Type.Enum.forString("spline").equals(type))
            throw new XmlException("The file " + splineXml
                    + " doesn't contain spline data");

    }
    /**
     * read the path data
     * @throws XmlException
     */
    private boolean getPathData()
    {
        final RMatrix3d R = new RMatrix3d();
        boolean isValid = true;
        
        final SplineType[] path = splineDoc.getPathConfiguration()
                .getSplinePathArray();
        nPath = path.length;

        QuaternionType quaternion = path[0].getQuaternion();
        MatrixType matrix = path[0].getMatrix();
        position = new Matrix(spaceDim + 1, nPath);

        if(!noOrientation)
            orientation = new Matrix(5, nPath);

        // System.out.println(nPath);
        for (int i = 0; i < nPath; i++)
        {
            // int id = path[i].getId() - 1;
            final double time = path[i].getTime();
            final double[] points = path[i].getPosition().getPointArray();

            // position
            position.setElement(0, i, time);

            for (int j = 0; j < spaceDim; j++)
                position.setElement(j + 1, i, points[j]);

            // orientation
            if(!noOrientation)
            {
                quaternion = path[i].getQuaternion();
                matrix = path[i].getMatrix();
                orientation.setElement(0, i, time);
                
                if(quaternion != null)
                {
                    orientation.setElement(1, i, quaternion.getScalar());
                    orientation.setElement(2, i, quaternion.getVectorX());
                    orientation.setElement(3, i, quaternion.getVectorY());
                    orientation.setElement(4, i, quaternion.getVectorZ());
                }
                else
                    if(matrix != null)
                    {
                        R.m00 = matrix.getR11();
                        R.m01 = matrix.getR12();
                        R.m02 = matrix.getR13();
                        R.m10 = matrix.getR21();
                        R.m11 = matrix.getR22();
                        R.m12 = matrix.getR23();
                        R.m20 = matrix.getR31();
                        R.m21 = matrix.getR32();
                        R.m22 = matrix.getR33();

                        final Quaternion tempQuat = R.getQuaternion();

                        orientation.setElement(1, i, tempQuat.w);
                        orientation.setElement(2, i, tempQuat.x);
                        orientation.setElement(3, i, tempQuat.y);
                        orientation.setElement(4, i, tempQuat.z);
                    }
                    else
                    {
                        isValid = false;
                    }
            }
        }
        return isValid;
    }

    /**
     * Gets the tag description of the file.
     * 
     * @return the tag description value.
     */
    final public String getDescription()
    {
        return description;
    }
    
    /**
     * Gets the tag name of the file.
     * 
     * @return the tag name value.
     */
    final public String getName()
    {
        return name;
    }
    /**
     * Gets orientation path data. The i-th column of matrix contains a couple
     * [t<sub>i</sub>;Quaternion(t<sub>i</sub>)].
     * 
     * @return the 5-by-nPath matrix of orientation data.
     */
    final public Matrix getOrientationPathAsMatrix()
    {
        return orientation;
    }

    /**
     * Gets position path data. The i-th column of matrix contains a couple [t<sub>i</sub>;P(t<sub>i</sub>)].
     * 
     * @return the spaceDim+1-by-nPath matrix of position data.
     */
    final public Matrix getPositionPathAsMatrix()
    {
        return position;
    }

    /**
     * Gets the number of points of path
     * 
     * @return the number of points.
     */
    final public int getNumberOfPoints()
    {
        return nPath;
    }

    /**
     * Gets tag orientation of the file.
     * 
     * @return the tag orientation value.
     */
    final public boolean hasOrientation()
    {
        return !noOrientation;
    }

    /**
     * Gets the handle of xml configuration file.
     * 
     * @return the xml handler.
     */
    final public PathConfigurationType getXmlHandler()
    {
        return splineDoc.getPathConfiguration();
    }

}
