/*
 * XmlTrapezoidal.java -
 * Copyright (c) 2005 Carmine Lia
 * e-mail:      carmine.lia@libero.it
 * web-address: http://www.intermedia.sa.it/lia

 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
package javax.robotics.xml;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;

import javax.robotics.vecmath.RVector;

import org.apache.xmlbeans.XmlException;

import pathML.PathConfigurationDocument;
import pathML.PathConfigurationType;
import pathML.TrapezoidalPointType;
import pathML.TrapezoidalType;


/**
 * 
 * XML handler class for trapezoida path configuration file.
 * 
 * 
 * @author <a href="http://www.intermedia.sa.it/lia/">Carmine Lia</a>
 * @since 1.1.0
 * @version 30/10/2005
 */
public class XmlTrapezoidal
{

    private PathConfigurationDocument trapDoc = null;
    private File trapXml;
    private InputStream trapXmlStream;
    private String description, name;
    private pathML.PathConfigurationType.Type.Enum type;
    private RVector ps, pf, vMax, tf;
    private int nPath, spaceDim;
    private boolean noOrientation = true;

    /**
     * Constructor of the xml handler.
     * @param trapXml the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlTrapezoidal(String trapXml) throws XmlException, IOException
    {
        this(new File(trapXml));
    }
    
    /**
     * Constructor of the xml handler.
     * @param trapXml the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlTrapezoidal(File trapXml) throws XmlException, IOException
    {
        this.trapXml = trapXml;
        trapDoc = PathConfigurationDocument.Factory.parse(this.trapXml);
        if(!trapDoc.validate())
            throw new XmlException("The file " + trapXml.getName() + " isn't a valid xml");

        getGeneralData();
        getPathData();
    }
    /**
     * Constructor of the xml handler.
     * @param trapXml the xml configuration file.
     * @throws XmlException
     * @throws IOException
     */
    public XmlTrapezoidal(InputStream trapXml) throws XmlException, IOException
    {
        this.trapXmlStream = trapXml;
        trapDoc = PathConfigurationDocument.Factory.parse(this.trapXmlStream);
        if(!trapDoc.validate())
            throw new XmlException("The input stream isn't a valid xml");

        getGeneralData();
        getPathData();
    }
    /**
     * read the information about path
     * @throws XmlException
     */
    private void getGeneralData() throws XmlException
    {
        description = trapDoc.getPathConfiguration().getDescription();
        name = trapDoc.getPathConfiguration().getName();
        type = trapDoc.getPathConfiguration().getType();
        spaceDim = trapDoc.getPathConfiguration().getSpaceDimension();
        noOrientation = !trapDoc.getPathConfiguration().getOrientation();
        
        if(!pathML.PathConfigurationType.Type.Enum.forString("trapezoidal").equals(type))
            throw new XmlException("The file " + trapXml
                    + "doesn't contain trapezoidal data");

    }

    // 
    /**
     * read the path data
     * @throws XmlException
     */
    private void getPathData() throws XmlException
    {
        ps = new RVector(spaceDim);
        pf = new RVector(spaceDim);
        vMax = new RVector(spaceDim);
        tf = new RVector(spaceDim);
        
        final TrapezoidalType[] path = trapDoc.getPathConfiguration()
                .getTrapezoidalPathArray();
        nPath = path.length;
        if(nPath == 2)
        {
            // gets initial positions
            TrapezoidalPointType[] points = path[0].getInitialPositionArray();

            for (int i = 0; i < spaceDim; i++)
            {
                ps.setElement(i, points[i].getPoint());
                vMax.setElement(i, points[i].getMaxVelocity());
            }
            // gets final position
            points = path[1].getFinalPositionArray();
            for (int i = 0; i < spaceDim; i++)
            {
                pf.setElement(i, points[i].getPoint());
                tf.setElement(i, points[i].getTime());
            }
        }
        else
            throw new XmlException("I can't handle the intermediate points");

    }

    /**
     * Gets the tag description of the file.
     * @return the tag description value.
     */
    final public String getDescription()
    {
        return description;
    }

    /**
     * Gets the tag name of the file.
     * 
     * @return the tag name value.
     */
    final public String getName()
    {
        return name;
    }
    
    /**
     * Gets tag orientation of the file.
     * @return the tag orientation value.
     */
    final public boolean isOrientationSet()
    {
        return !noOrientation;
    }

    /**
     * Gets the handle of xml configuration file.
     * @return the xml handler.
     */
    final public PathConfigurationType getXmlHandler()
    {
        return trapDoc.getPathConfiguration();
    }

    /**
     * Gets the final positions of the path.
     * @return the final position.
     */
    final public RVector getFinalPosition()
    {
        return pf;
    }

    /**
     * Gets the initial positions of the path.
     * @return the initial positions.
     */
    final public RVector getInitialPosition()
    {
        return ps;
    }

    /**
     * Gets the final times of the path.
     * @return the initial positions.
     */
    final public RVector getFinalTime()
    {
        return tf;
    }

    /**
     * Gets the maximum velocities of the path.
     * @return the maximum velocities.
     */
    final public RVector getMaximumVelocity()
    {
        return vMax;
    }

}
